#include <stdio.h>
#include <time.h>
#include "cassl.h"

#include <openssl/bio.h>
#include <openssl/err.h>
#include <openssl/ssl.h>
#include <openssl/buffer.h>
#include <openssl/objects.h>
#include <openssl/evp.h>
#include <openssl/rand.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/pem.h>
#include <openssl/pkcs12.h>
#include <openssl/des.h>


int main(int argc, char **argv)
{
     X509_REQ *req;
     X509 *crt;
     char *z;
     char buf[10240];
     char *r;
     int i;
     time_t t;
     char **alt;
     char *pemfile;
     char *id = NULL;


     FILE *f;

     if(argc==2) {
        pemfile = argv[1];
     } else if (argc==3) {
        pemfile = argv[1];
        id = argv[2];
     } else {
        fprintf(stderr, "Usage %s pem_file [id]\n");
        exit(1);
     }
  
     if(!(f = fopen(pemfile, "r"))) {
        fprintf(stderr, "Can't open %s\n", pemfile);
        exit(1);
     }
     fread(buf, 1, 10240, f);
     fclose(f);


     if (strstr(buf,"REQUEST")) {
        char *err;
        X509_NAME *subj;
        X509_NAME_ENTRY *ne;
        int i,n,l,l1,j,l2;
        const char *s;
        int type, num;
        unsigned char *q;
        int gs_doit[4];
        char tmp_buf[80];
        static char hex[17]="0123456789ABCDEF";

        char pbuf[16384];
        char *p;
        
        req = ssl_txt2req(buf);

        subj = X509_REQ_get_subject_name(req);

        for (i=0; i<sk_X509_NAME_ENTRY_num(subj->entries); i++) {
           ne=sk_X509_NAME_ENTRY_value(subj->entries,i);
           n=OBJ_obj2nid(ne->object);
           s=OBJ_nid2sn(n);
           l1=strlen(s);
           type=ne->value->type;
           num=ne->value->length;
           q=ne->value->data;

               if ((type == V_ASN1_GENERALSTRING) && ((num%4) == 0)) {
                        gs_doit[0]=gs_doit[1]=gs_doit[2]=gs_doit[3]=0;
                        for (j=0; j<num; j++) if (q[j] != 0) gs_doit[j&3]=1;

                        if (gs_doit[0]|gs_doit[1]|gs_doit[2]) gs_doit[0]=gs_doit[1]=gs_doit[2]=gs_doit[3]=1;
                        else {
                                gs_doit[0]=gs_doit[1]=gs_doit[2]=0;
                                gs_doit[3]=1;
                        }
               } else gs_doit[0]=gs_doit[1]=gs_doit[2]=gs_doit[3]=1;


                for (l2=j=0; j<num; j++) {
                        if (!gs_doit[j&3]) continue;
                        l2++;
                        if ((q[j] < ' ') || (q[j] > '~')) l2+=3;
                }

                l+=1+l1+1+l2;

                p = pbuf;
                *(p++)='/';

                memcpy(p,s,(unsigned int)l1); p+=l1;
                *(p++)='=';

                q=ne->value->data;

                for (j=0; j<num; j++) {
                        if (!gs_doit[j&3]) continue;
                        n=q[j];
                        if ((n < ' ') || (n > '~')) {
                                *(p++)='\\';
                                *(p++)='x';
                                *(p++)=hex[(n>>4)&0x0f];
                                *(p++)=hex[n&0x0f];
                        } else *(p++)=n;
                }
                *p='\0';
                printf("part: %s\n", pbuf);
           
                printf("nozeros: ");
                for (j=0;j<num;j++) {
                    n = q[j];
                    if (n) printf("%c", n);
                }
                printf("\n");
           }

        // ssl_rewrite_req_dn(req);

        r = ssl_get_dn_req(req);
        printf("Req: DN=%s  ", r);
        if (id) {
           if (err=ssl_chk_req_dn(req, id)) printf("<< %s >>\n", err);
           else printf("OKDN\n");
        }

        r = ssl_get_cn_req(req);
        printf("     CN=%s  ", r);
        if (err=ssl_chk_dns(r, 0)) printf("<< %s >>\n", err);
        else printf("is in DNS\n");
        if (id) {
           if (err = ssl_chk_netacts(r, 0, id)) {
              printf("         << %s >>\n", err);
           } else {
              printf("         owned by %s\n",id);
           }
        }
           

        for (alt=ssl_get_altnames_req(req);alt&&*alt;alt++) {
           printf("    Alt=%s  ", *alt);
           if (err=ssl_chk_dns(*alt, 0)) printf("<< %s >>\n", err);
           else printf("is in DNS\n");
           if (id) {
              if (err = ssl_chk_netacts(*alt, 0, id)) {
                 printf("         << %s >>\n", err);
              } else {
                 printf("         owned by %s\n",id);
              }
           }
        }
        if (id) {
          if (err=ssl_chk_req_altnames(req, id)) {
             printf("         << altnames not owned by %s: %s\n", id, err);
          } else {
             printf("         altnames owned by %s\n",id);
          }
        }

        ssl_get_keyusage_req(req);


     } else if (strstr(buf,"CERTIFICATE")) {
        crt = ssl_txt2crt(buf);
        r = ssl_get_dn(crt);
        printf("Cert: DN=%s\n", r);
        i = ssl_get_sn(crt);
        printf("      SN=%d\n", i);
        t = ssl_get_expire(crt);
        printf("      EX=%s\n", ctime(&t));

     } else printf("not a cert or request\n");


}

