/* ca ssl tools */


#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <time.h>

#include <openssl/bio.h>
#include <openssl/err.h>
#include <openssl/ssl.h>
#include <openssl/buffer.h>
#include <openssl/objects.h>
#include <openssl/evp.h>
#include <openssl/rand.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/pem.h>
#include <openssl/pkcs12.h>
#include <openssl/des.h>


/* from x509 code */
/* Get a normal integer from an asn1 integer */

int i2i_ASN1_INTEGER(ASN1_INTEGER *a)
{
  int i;
  int h1, h2;
  int ret = 0;

  for (i=0;i<a->length;i++) {
     h1 = ((unsigned char)a->data[i]>>4)&0x0f;
     h2 = ((unsigned char)a->data[i]   )&0x0f;
     ret = ret*256 + h1*16 + h2;
  }
  return (ret);
}


/* ASN1 to time_t ( from coded-out part of a_utctm.c) */

#define g2(p) (((p)[0]-'0')*10+(p)[1]-'0')
time_t ASN1_UTCTIME_get(ASN1_UTCTIME *s)
{
   struct tm tm;
   int offset;

   memset(&tm,'\0',sizeof tm);

   tm.tm_year=g2(s->data);
   if(tm.tm_year < 50) tm.tm_year+=100;
   tm.tm_mon=g2(s->data+2)-1;
   tm.tm_mday=g2(s->data+4);
   tm.tm_hour=g2(s->data+6);
   tm.tm_min=g2(s->data+8);
   tm.tm_sec=g2(s->data+10);
   if(s->data[12] == 'Z') offset=0;
   else {
      offset=g2(s->data+13)*60+g2(s->data+15);
      if(s->data[12] == '-')
              offset= -offset;
   }
   return (mktime(&tm)-offset*60);
}
#undef g2

/* --------- end of openssl x509 extraction ----------- */

/* Get a request from a string */

X509_REQ *ssl_txt2req(char *reqstr)

{
  X509_REQ *req=NULL;
  BIO *rbio;
   
  rbio = BIO_new_mem_buf(reqstr, strlen(reqstr));
  if (rbio==NULL) {
    ERR_print_errors_fp(stderr);
    return (NULL);
  }

  if(!(req = PEM_read_bio_X509_REQ(rbio, NULL, NULL, NULL))) {
      fprintf(stderr, "Can't parse request %s\n", reqstr);
      return (NULL);
  }

  /* close rbio */

  return (req);
}


/* Get the dn from a request */

char *ssl_get_dn_req(X509_REQ *req)

{
  X509_NAME *subj;
  subj = X509_REQ_get_subject_name(req);
  return (X509_NAME_oneline(subj,NULL,0));

}

/*** have sub fro this */
/* return the cn part of a dn */
char *ssl_dn2cn(char *dn)
{
  char *cn;
  char *e;

  e = strstr(dn, "CN=");
  if (e) {
    cn = strdup(e+3);
    if (e=strchr(cn,'/')) *e = '\0';
    return (cn);
  }
  return (dn);
}


 

/* Get a cert from a string */

X509 *ssl_txt2crt(char *crtstr)
{
  X509 *crt=NULL;
  BIO *cbio;

  cbio = BIO_new_mem_buf(crtstr, strlen(crtstr));
  if (cbio==NULL) {
    ERR_print_errors_fp(stderr);
    return (NULL);
  }

  if(!(crt = PEM_read_bio_X509(cbio, NULL, NULL, NULL))) {
      fprintf(stderr, "Can't parse cert %s\n", crtstr);
      return (NULL);
  }
  return (crt);
}


/* Get the dn from a cert */

char *ssl_get_dn(X509 *crt)
{
  X509_NAME *subj;
  subj = X509_get_subject_name(crt);
  return (X509_NAME_oneline(subj,NULL,0));
}

/* Get the serial from a cert */

int ssl_get_sn(X509 *crt)
{
  X509_NAME *subj;
  int sn;
  sn = i2i_ASN1_INTEGER(crt->cert_info->serialNumber);
  return (sn);

}

/* Get the issue date from a cert */

time_t ssl_get_issue(X509 *crt)
{
  X509_NAME *subj;
  ASN1_UTCTIME *str;
  str = X509_get_notBefore(crt);
  return (ASN1_UTCTIME_get(str));
}


/* Get the expire date from a cert */

time_t ssl_get_expire(X509 *crt)
{
  X509_NAME *subj;
  ASN1_UTCTIME *expstr;
  expstr = X509_get_notAfter(crt);
  return (ASN1_UTCTIME_get(expstr));
}

 

/* Convert a cert to der encoding */

char *ssl_crttxt2der(char *crttxt, int *len)
{
  X509 *crt;
  BIO *dbio;
  int ret;
  BUF_MEM *derbuf;
  char *dertxt;

  crt = ssl_txt2crt(crttxt);
  if (!crt) return (NULL);
  dbio = BIO_new(BIO_s_mem());
  if (dbio==NULL) {
    ERR_print_errors_fp(stderr);
    return (NULL);
  }
  ret = BIO_get_mem_ptr(dbio, &derbuf);
  ret = i2d_X509_bio(dbio, crt);
  if (!ret) {
    ERR_print_errors_fp(stderr);
    return (NULL);
  }

  dertxt = (char*) malloc(derbuf->length);
  memcpy(dertxt, derbuf->data, derbuf->length);
  *len = derbuf->length;
  return (dertxt);
}


/* ------------------------------- */

