#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include "caserver.h"


#define CM_LIST   1
#define CM_REVOKE 2
#define CM_VOID   3
#define CM_ISSUE  4
#define CM_EXPIRE 5
#define CM_DN     6
#define CM_EXPORT 7
#define CM_IMPORT 8

#define ARG(a) (!strcmp(argv[0],a))

char *prog = "";
int crtdb;

int usage() {
  fprintf(stderr,"usage: %s -list [-sn serial]\n",prog);
  fprintf(stderr,"       %s -void -sn serial\n",prog);
  fprintf(stderr,"       %s -dn -sn serial [-name dn_text] \n",prog);
  fprintf(stderr,"       %s -revoke -sn serial [-date time_t] \n",prog);
  fprintf(stderr,"       %s -expire -sn serial [-date time_t] \n",prog);
  fprintf(stderr,"       %s -issue -sn serial [-date time_t] \n",prog);
  fprintf(stderr,"       %s -export exp_file \n",prog);
  fprintf(stderr,"       %s -import imp_file \n",prog);
  exit(1);
}

char *ymd(time_t t) {
  char *ret = (char*)malloc(24);
  struct tm *d = localtime(&t);
  if (t==0) return("--/--/--");
  sprintf(ret, "%2d/%02d/%04d",
     d->tm_mon+1, d->tm_mday, d->tm_year+1900);
  return (ret);
}

void show(CertRec C)
{
   if (C->casn) printf(" %4x  %s %s %s %s\n", C->casn,
        ymd(C->issue_date),
        ymd(C->expire_date),
        ymd(C->revoke_date),
        C->dn);
   else printf(" %4x  --void--\n", C->casn);
}

int getcrt(int sn, CertRec C)
{
   if (lseek(crtdb, sn*sizeof(CertRec_), SEEK_SET)<0) {
       perror("seek");
       return (0);
   }
   if (read(crtdb, C, sizeof(CertRec_))<=0) {
       perror("read");
       return (0);
   }
   return (1);
}

int putcrt(int sn, CertRec C)
{
   if (lseek(crtdb, sn*sizeof(CertRec_), SEEK_SET)<0) {
       perror("seek");
       return (0);
   }
   if (write(crtdb, C, sizeof(CertRec_))<=0) {
       perror("write");
       return (0);
   }
   return (1);
}

char *nxtstr(char **ptr)
{
  char *p = *ptr;
  char *e;
  while (isspace(*p)) p++;
  e = p;
  while (*e && !isspace(*e)) e++;
  *e++ = '\0';
  *ptr = e;
  return (p);
}
  

main(int argc, char **argv)
{
  CertRec C = (CertRec) malloc(sizeof(CertRec_));
  int sn = (-1);
  int nb;
  time_t now = time(NULL);
  struct tm *t = localtime(&now);

  time_t dt = time(NULL);
  int cmd = 0;
  char *name = NULL;
  char *expname = NULL;
  FILE *expfile = NULL;
  int nrec = 0;
  char impbuf[1024];
  
  prog = argv[0];

  crtdb = open(CA_CERTDB, O_CREAT|O_RDWR, 0660);
  if (!crtdb) {
     perror(CA_CERTDB);
     exit(1);
  }

  while (--argc) {
     argv++;
     if (ARG("-sn")) {
        if (--argc<-0) usage();
        ++argv;
        sn = strtol(argv[0], NULL, 16);
     }
     if (ARG("-date")) {
        if (--argc<-0) usage();
        ++argv;
        dt = atoi(argv[0]);
     }
     if (ARG("-name")) {
        if (--argc<-0) usage();
        ++argv;
        name = argv[0];
     }
     if (ARG("-list")) {
        cmd = CM_LIST;
     }
     if (ARG("-dn")) {
        cmd = CM_DN;
     }
     if (ARG("-void")) {
        cmd = CM_VOID;
     }
     if (ARG("-expire")) {
        cmd = CM_EXPIRE;
     }
     if (ARG("-revoke")) {
        cmd = CM_REVOKE;
     }
     if (ARG("-issue")) {
        cmd = CM_ISSUE;
     }
     if (ARG("-export")) {
        if (--argc<-0) usage();
        ++argv;
        expname = argv[0];
        cmd = CM_EXPORT;
     }
     if (ARG("-import")) {
        if (--argc<-0) usage();
        ++argv;
        expname = argv[0];
        cmd = CM_IMPORT;
     }
  }
  
  switch (cmd) {
    case CM_LIST: 
        printf("   sn   issued     expires   revoked   dn\n");
        if (sn<0) {
           while (read(crtdb, C, sizeof(CertRec_))>0) show(C);
        } else {
           if (getcrt(sn, C)) show(C);
        }
        break;
     
    case CM_DN:
        if (sn<0) usage();
        if (getcrt(sn, C)) {
           name[239] = '\0';
           strcpy(C->dn,name);
           C->casn = sn;
           putcrt(sn,C);
        }
        break;

    case CM_VOID:
        if (sn<0) usage();
        if (getcrt(sn, C)) {
           C->casn = 0;
           putcrt(sn,C);
        }
        break;
         
    case CM_REVOKE:
        if (sn<0) usage();
        if (getcrt(sn, C)) {
           C->revoke_date = dt;
           C->casn = sn;
           putcrt(sn,C);
        }
        break;
         
    case CM_ISSUE:
        if (sn<0) usage();
        if (getcrt(sn, C)) {
           C->issue_date = dt;
           C->casn = sn;
           putcrt(sn,C);
        }
        break;
         
    case CM_EXPIRE:
        if (sn<0) usage();
        if (getcrt(sn, C)) {
           C->expire_date = dt;
           C->casn = sn;
           putcrt(sn,C);
        }
        break;

    case CM_EXPORT: 
        if (!(expfile=fopen(expname,"w"))) {
           perror(expname);
           exit(1);
        }
        while (read(crtdb, C, sizeof(CertRec_))>0) {
           fprintf(expfile," %x %u %u %u %s\n", C->casn,
               C->issue_date, C->expire_date, C->revoke_date, C->dn);
           sn = C->casn;
           nrec++;
        }
        printf(" %d records exported, last sn = %d\n", nrec, sn);
        break;
     
    case CM_IMPORT: 
        if (!(expfile=fopen(expname,"r"))) {
           perror(expname);
           exit(1);
        }
        while (fgets(impbuf, 1024, expfile)) {
           char *p = impbuf;
           C->casn = strtol(nxtstr(&p), NULL, 16);
           C->issue_date = atoi(nxtstr(&p));
           C->expire_date = atoi(nxtstr(&p));
           C->revoke_date = atoi(nxtstr(&p));
           while (isspace(*p)) p++;
           strcpy(C->dn, p);
           if (p=strchr(C->dn,'\n')) *p = '\0';
           sn = C->casn;
           if (sn<10000) putcrt(sn, C);
           nrec++;
        }
        printf(" %d records imported, last sn = %d\n", nrec, sn);
        break;
     
     default: usage();
  }
         
  close(crtdb);
  exit (0);
        
}
