/* C&C CA client.

   Command line CA tool.
   Contacts remote server to sign requests, create CRLs

   by Fox, March 2003

*/


#include <stdio.h>
#include <string.h>

#include "caclient.h"

extern int  debug;

extern char *caport;
extern char *cahost;

/* ------------------------------------------------- */


char *prog;

void usage()
{
   fprintf(stderr, "usage: %s -c sign|revoke|getcrl|pkcs7 -u userid \n", prog);
   fprintf(stderr, "          [-d] [-e s|c|sc]  \n");
   fprintf(stderr, "          [-sn revoke_sn] [-dn revoke_dn ]\n");
   fprintf(stderr, "          [-a reqfile] \n");
   fprintf(stderr, "          [-t timeout] [-o outfile]\n");
   fprintf(stderr, "          [-p port] [-h host]\n");
   exit(-1);
}




int  main(int argc, char *argv[])
{
   char   *p;
   char *arg;
   struct timeval tv;
   int  interval = 0;
   char *s;
   uint32_t cmd = 0;
   char *id = NULL;
   char cmdarg[CA_MAXARG]; 
   char *argfile = NULL;
   FILE *af;
   int ret;
   char *retarg = NULL;
   int nb;
   char *certpos;
   char *exttxt = NULL;
   uint32_t extcode = CA_EXT_SV;
   int casn = 0;
   int timeout;
   FILE *binfile = NULL;
   char *rvkdn = NULL;
   uint32_t flags = CA_FL_ALTNAMDNS;
  

   /* parse command line options */

   prog = argv[0];
   while (--argc) {
      arg = (++argv)[0];

      if (!strcmp(arg,"-v")) {
            debug = 1;

      } else if (!strcmp(arg,"-d")) {
            debug = 2;

      } else if (!strcmp(arg,"-c")) {
            if (--argc<0) usage();
            s = ((++argv)[0]);
            if (strcmp(s,"sign")==0) cmd = CA_SIGN;
            else if (strcmp(s,"revoke")==0) cmd = CA_REVOKE;
            else if (strcmp(s,"gencrl")==0) cmd = CA_GENCRL;
            else if (strcmp(s,"getcrl")==0) cmd = CA_GETCRL;
            else if (strcmp(s,"pkcs7")==0) cmd = CA_PKCS7;
            else usage();

      } else if (!strcmp(arg,"-sn")) {
            if (--argc<0) usage();
            s = ((++argv)[0]);
            casn = atoi(s);
            if (casn<=0) usage();

      } else if (!strcmp(arg,"-dn")) {
            if (--argc<0) usage();
            rvkdn = ((++argv)[0]);

      } else if (!strcmp(arg,"-u")) {
            if (--argc<0) usage();
            id = ((++argv)[0]);

      } else if (!strcmp(arg,"-a")) {
            if (--argc<0) usage();
            argfile = ((++argv)[0]);

      } else if (!strcmp(arg,"-e")) {
            if (--argc<0) usage();
            exttxt = ((++argv)[0]);
            if (!strcmp(exttxt,"s")) extcode = CA_EXT_SV;
            else if (!strcmp(exttxt,"c")) extcode = CA_EXT_CL;
            else if (!strcmp(exttxt,"sc")) extcode = CA_EXT_SC;
            else usage();

      } else if (!strcmp(arg,"-h")) {
            if (--argc<0) usage();
            cahost = ((++argv)[0]);

      } else if (!strcmp(arg,"-p")) {
            if (--argc<0) usage();
            caport = ((++argv)[0]);
            if (atoi(caport)<=0) usage();

      } else if (!strcmp(arg,"-t")) {
            if (--argc<0) usage();
            s = ((++argv)[0]);
            timeout = atoi(s);
            if (timeout<=0) usage();

      } else if (!strcmp(arg,"-o")) {
            if (--argc<0) usage();
            s = ((++argv)[0]);
            binfile = fopen(s,"w");
            if (!binfile) {
               perror("binfile");
               exit (1);
            }
      } else usage();
   }

   if (!id || !cmd) usage();
   if (strlen(id)>19) usage();

   /* Sign */
   if (cmd == CA_SIGN || cmd==CA_PKCS7) {
     cmdarg[0] = '\0';
     nb = 0;
     if (argfile) {
        af = fopen(argfile,"r");
        if (!af) {
           perror(argfile);
           return (-1);
        }
     } else af = stdin;
     nb = fread(cmdarg, 1, CA_MAXARG, af);
     cmdarg[nb] = '\0';
     fclose(af);
     if (nb<24) usage();

     ret = ca_cmd(cmd, id, cmdarg, extcode, flags, &retarg);

     if (retarg) {
        certpos = strstr(retarg,"-----BEGIN ");
        if (!certpos) {
           certpos = retarg;
           if (!ret) ret = 1;
        }
        if (binfile) fputs(certpos, binfile);
        else puts(certpos);
     }
   } else if (cmd == CA_REVOKE) {

     if (!(casn&&rvkdn)) usage();
     ret = ca_cmd(cmd, id, rvkdn, casn, 0, &retarg);

     if (retarg) {
        if (binfile) fputs(retarg, binfile);
        else puts(retarg);
     }

   } else if (cmd==CA_GETCRL||cmd==CA_GENCRL) {

     ret = ca_cmd(cmd, id, "void", 0, 0, &retarg);

     if (retarg) {
        if (binfile) fputs(retarg, binfile);
        else puts(retarg);
     }

   }
   exit (ret?ret:0);
}
