/* ca database routines (DB2 version) */

/**
 
    These have not been kept up-to-date.
    They are here only to show how db2 might be accessed.

 **/

#include <string.h>
#include <stdio.h>
#include <time.h>

#include <sqlcli1.h>

#define UWCA_db_lib
#include "db.h"
#include "util.h"

extern char *remote_user;

static SQLRETURN    rc;
void sql_abort(char *c)
{
   printf("db fails: %d: %s\n", rc, c);
}

void sql_errprint(SQLSMALLINT htype, SQLHANDLE h, SQLRETURN rc)
{
  SQLCHAR buf[SQL_MAX_MESSAGE_LENGTH+1];
  SQLCHAR state[SQL_SQLSTATE_SIZE+1];
  SQLINTEGER code;
  SQLSMALLINT len, i=1;
  
  printf("SQL ERR: rc = %d\n", rc);
  while (SQLGetDiagRec(htype, h, i, state, &code,
            buf, SQL_MAX_MESSAGE_LENGTH+1, &len) == SQL_SUCCESS) {
      printf("   state: %s\n", state);
      printf("   error: %ld\n", code);
      printf("          %s\n", buf);
      i++;
  }
}


SQLHANDLE    henv;
SQLHANDLE    hdbc;
SQLHANDLE    h_req;
SQLHANDLE    h_crt;
SQLHANDLE    h_not;

SQLHANDLE    h_xxx;

static char *DB2_ENV = "DB2INSTANCE=db2inst1"; /* Database user env */
char dsn[20] = "certs";                        /* Data Source Name  */

static int connected = 0;

/* Establish a db connection */

int db_connect() {

   if (connected) return;

   /* DB2 library needs the db2 environment variable */

   putenv(DB2_ENV);


/*  Allocate environment handle, allocat connection handle, and connect  */
   rc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
   if (rc!=SQL_SUCCESS) sql_abort("SQLAllocHandle(ENV)");

   rc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
   if (rc!=SQL_SUCCESS) sql_abort("SQLAllocHandle(DBC)");

   rc = SQLConnect(hdbc, dsn, SQL_NTS, "db2uas", SQL_NTS, "r5t6y7u8", SQL_NTS);
   if (rc!=SQL_SUCCESS) sql_abort("SQLConnect");

   /* printf( "ODBC connection to %s successful.\n", dsn);  */

/*  Allocate a statement handle. */
   rc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &h_req);
   if (rc!=SQL_SUCCESS) sql_abort("SQLAllocHandle(STMT)");

   rc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &h_crt);
   if (rc!=SQL_SUCCESS) sql_abort("SQLAllocHandle(STMT)");

   rc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &h_not);
   if (rc!=SQL_SUCCESS) sql_abort("SQLAllocHandle(STMT)");

   rc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &h_xxx);
   if (rc!=SQL_SUCCESS) sql_abort("SQLAllocHandle(STMT)");

   connected = 1;
}

void db_disconnect()
{
   if (!connected) return;
   SQLFreeHandle(SQL_HANDLE_STMT, h_req);
   SQLFreeHandle(SQL_HANDLE_STMT, h_crt);
   SQLFreeHandle(SQL_HANDLE_STMT, h_not);
   SQLFreeHandle(SQL_HANDLE_STMT, h_xxx);
   SQLDisconnect(hdbc);
   SQLFreeConnect(hdbc);
   SQLFreeHandle(SQL_HANDLE_ENV, henv);
   fprintf(stderr, "ODBC connection to %s closed.\n", dsn);
   connected = 0;
}


/* ------------ DB retrieval functions -------------- */


static time_t build_time(struct DATE_STRUCT *d, struct TIME_STRUCT *t)
{
    struct tm bldtm;
    memset (&bldtm,'\0', sizeof(bldtm));
    bldtm.tm_year = d->year - 1900;
    bldtm.tm_mon = d->month - 1;
    bldtm.tm_mday = d->day;
    if (t) {
      bldtm.tm_hour = t->hour;
      bldtm.tm_min = t->minute;
      bldtm.tm_sec = t->second;
    }
    bldtm.tm_isdst = (-1);
    return mktime(&bldtm);
}


/* make a db2 style date */

static char *db_fix_date(time_t d)
{
   static char ret[12];
   struct tm *t;
   t = localtime(&d);
   sprintf(ret, "%d-%02d-%02d", t->tm_year+1900, t->tm_mon+1, t->tm_mday);
   return(ret);
}



   
/* Retrieve a selected record */

int db_get_req(SvrReq R)
{
   int rc;
   char *b;
   struct DATE_STRUCT xdate;

   rc = SQLFetch(h_req);

   if (rc == SQL_NO_DATA_FOUND) {
      /* fprintf(stderr, "fetch eof\n"); */
      return (0);
   } else if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
      fprintf(stderr, "fetch err = %d\n", rc);
      return (-1);
   }
 
   memset(R, '\0', sizeof(SvrReq_));
   rc = SQLGetData(h_req, 1, SQL_C_LONG, &R->reqno, sizeof(R->reqno), NULL);
   rc = SQLGetData(h_req, 2, SQL_C_CHAR, &R->reqid, sizeof(R->reqid), NULL);
   if (b=strchr(R->reqid,' ')) *b = '\0';
   rc = SQLGetData(h_req, 3, SQL_C_LONG, &R->status, sizeof(R->status), NULL);
   rc = SQLGetData(h_req, 4, SQL_C_TYPE_DATE, &xdate, sizeof(xdate), NULL);
   R->request_date = build_time(&xdate, NULL);
   rc = SQLGetData(h_req, 5, SQL_C_LONG, &R->crtno, sizeof(R->crtno), NULL);
   rc = SQLGetData(h_req, 6, SQL_C_CHAR, &R->request, sizeof(R->request), NULL);
   rc = SQLGetData(h_req, 8, SQL_C_LONG, &R->hash, sizeof(R->hash), NULL);

   /* has to end on a positive note */
   rc = SQLGetData(h_req, 7, SQL_C_CHAR, &R->dn, sizeof(R->dn), NULL);
   return (rc==SQL_SUCCESS);
}


int db_get_crt(SvrCert S)
{
   int rc;
   struct DATE_STRUCT xdate;

   rc = SQLFetch(h_crt);

   if (rc == SQL_NO_DATA_FOUND) {
      /* fprintf(stderr, "fetch eof\n"); */
      return (0);
   } else if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
      sql_errprint(SQL_HANDLE_STMT, h_crt, rc);
      fprintf(stderr, "fetch err = %d\n", rc);
      return (-1);
   }
 
   memset(S, '\0', sizeof(SvrCert_));
   rc = SQLGetData(h_crt, 1, SQL_C_LONG, &S->crtno, sizeof(S->crtno), NULL);
   rc = SQLGetData(h_crt, 3, SQL_C_LONG, &S->casn, sizeof(S->casn), NULL);
   rc = SQLGetData(h_crt, 4, SQL_C_LONG, &S->status, sizeof(S->status), NULL);
   rc = SQLGetData(h_crt, 5, SQL_C_CHAR, &S->cert, sizeof(S->cert), NULL);

   rc = SQLGetData(h_crt, 6, SQL_C_TYPE_DATE, &xdate, sizeof(xdate), NULL);
   if (rc==SQL_SUCCESS) S->issue_date = build_time(&xdate, NULL);
   rc = SQLGetData(h_crt,7,SQL_C_CHAR,&S->issue_by,sizeof(S->issue_by),NULL);

   rc = SQLGetData(h_crt, 8, SQL_C_TYPE_DATE, &xdate, sizeof(xdate), NULL);
   if (rc==SQL_SUCCESS) S->revoke_date = build_time(&xdate, NULL);
   rc = SQLGetData(h_crt,9,SQL_C_CHAR,&S->revoke_by,sizeof(S->revoke_by),NULL);

   rc = SQLGetData(h_crt, 10, SQL_C_TYPE_DATE, &xdate, sizeof(xdate), NULL);
   if (rc==SQL_SUCCESS) S->expire_date = build_time(&xdate, NULL);
   rc = SQLGetData(h_crt, 11, SQL_C_LONG, &S->expno, sizeof(S->expno), NULL);

   /* has to end on a positive note */
   rc = SQLGetData(h_crt, 2, SQL_C_LONG, &S->reqno, sizeof(S->reqno), NULL);

   return (rc==SQL_SUCCESS);
}


/* Get a notes record */

int db_get_notes(SvrNotes N)
{
   int rc;

   rc = SQLFetch(h_not);

   if (rc == SQL_NO_DATA_FOUND) {
      /* fprintf(stderr, "fetch eof\n"); */
      return (0);
   } else if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
      fprintf(stderr, "fetch err = %d\n", rc);
      return (-1);
   }
 
   memset(N, '\0', sizeof(SvrNotes_));
   rc = SQLGetData(h_not, 1, SQL_C_LONG, &N->notno, sizeof(N->notno), NULL);
   rc = SQLGetData(h_not, 3, SQL_C_CHAR, &N->notes, sizeof(N->notes), NULL);
   /* has to end on a positive note */
   rc = SQLGetData(h_not, 2, SQL_C_LONG, &N->reqno, sizeof(N->reqno), NULL);
   return (rc==SQL_SUCCESS);
}


/* utility to assemble select's where clause */
static build_sel(char *sel, char **p)
{
   int f = 1;
   char *c = sel;

   while (*c) c++;
   while (*p) {
      if (f) strcat(strcpy(c, "where "), *p);
      else strcat(strcpy(c, " and "), *p);
      free (*p);
      while (*c) c++;
      p++;
      f = 0;
   }
   *c = '\0';
}


/* Select request records from the db.

   Can select by all, reqno, id, or status. */


int db_select_req(int reqno, char *id, int status, char *srtkey)
{
   char cmd[1024];
   char sel[1024];
   char srt[1024];
   char *srtp;
   int rc;
   char *selp[5];
   int seli = 0;

   /* Build the select */

   if (reqno) {
      selp[seli] = (char*) malloc(48);
      sprintf(selp[seli], " reqno = %d ", reqno);
      seli++;
   }  
   if (id && *id) {
      selp[seli] = (char*) malloc(48);
      sprintf(selp[seli], " reqid = '%s' ", id);
      seli++;
   }
   if (status==RST_ACTIVE) {
      selp[seli] = (char*) malloc(96);
      sprintf(selp[seli], " status=%d or status=%d or status=%d",
            ST_REQ, ST_REVREQ, ST_RENEW);
      seli++;
   } else if (status!=ST_NONE) {
      selp[seli] = (char*) malloc(96);
      sprintf(selp[seli], " status = %d ", status);
      seli++;
   }
   selp[seli] = NULL;
   sel[0] = '\0';
   build_sel(sel, selp);

   /* build sort */
  
   srtp = srt;
   while (srtkey && *srtkey) {
      int i = *srtkey - '0';
      if (i<1||i>5) break;
      if (srtp!=srt) *srtp++ = ',';
      strcpy(srtp, sort_vars[i]);
      while (*srtp) srtp++;
      srtkey++;
   }
   if (srtp==srt) strcpy(srt,"reqno");

   sprintf(cmd, "select * from svreq %s order by %s;", sel, srt);
   
   rc = SQLExecDirect(h_req, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_errprint(SQL_HANDLE_STMT, h_req, rc);
      sql_abort(cmd);
      return (0);
   }
   
   return (rc);
}



/* Select cert records from the db.
   
   Can select by crtno, reqno, and status. */


int db_select_crt(int crtno, int reqno, int status)
{
   char cmd[256];
   char sel[256];
   int rc;
   char *selp[4];
   int seli = 0;

   /* Build the select */

   if (crtno) {
      selp[seli] = (char*) malloc(48);
      sprintf(selp[seli], " crtno = %d ", crtno);
      seli++;
   }
   if (reqno) {
      selp[seli] = (char*) malloc(48);
      sprintf(selp[seli], " reqno = %d ", reqno);
      seli++;
   }  
   if (status) {
      selp[seli] = (char*) malloc(48);
      sprintf(selp[seli], " status = %d ", status);
      seli++;
   }
   selp[seli] = NULL;
   sel[0] = '\0';
   build_sel(sel, selp);

   sprintf(cmd, "select * from svcrt %s;", sel);
   

   rc = SQLExecDirect(h_crt, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_abort(cmd);
      return (0);
   }
   
   return (rc);
}


/* Select expiring cert records from the db. */
   

int db_select_crt_exp(int grace, int lim)
{
   char cmd[256];
   int rc;
   time_t t = time(NULL) + grace;

   sprintf(cmd,
   "select * from svcrt where status!=%d and expire_date<'%s' and expno<=%d;",
             CRT_EXPIRED, db_fix_date(t), lim);

   rc = SQLExecDirect(h_crt, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_abort(cmd);
      return (0);
   }
   
   return (rc);
}



/* Select notes records from the db.
   
   Can select by reqno. */


int db_select_notes(int reqno)
{
   char cmd[256];
   int rc;

   sprintf(cmd, "select * from svnot where reqno = %d;", reqno);
   
   rc = SQLExecDirect(h_not, cmd, SQL_NTS);
   if (rc && rc!=SQL_SUCCESS) {
      sql_errprint(SQL_HANDLE_STMT, h_not, rc);
      sql_abort(cmd);
      return (0);
   }
   
   return (rc);
}


/* Free the cursor associated with retrieval */

void db_free_req()
{
   SQLFreeStmt(h_req, SQL_CLOSE);
}

void db_free_crt()
{
   SQLFreeStmt(h_crt, SQL_CLOSE);
}

void db_free_notes()
{
   SQLFreeStmt(h_not, SQL_CLOSE);
}


/* ------------ DB loader functions -------------- */


/* fix a string so it can be loaded into db2
   - replace each quote with double quotes
   - replace newlines with db2-coded-newlines */

char *db_fix_string(char *cert)
{
  char *fcert;
  char *p;
  int nl = 0;
 
  /* malloc new string */
  for (fcert=cert;*fcert;fcert++,nl+=2) {
     if (*fcert=='\n') nl+=15;
     if (*fcert=='\'') nl+=2;
  }
  fcert = (char*) malloc(nl);

  p = fcert;
  *p++ = '\'';
  while (*cert) {
     if (*cert=='\'') *p++ = '\'';
     if (*cert!='\n') *p++ = *cert;
     else {
        strcpy(p, "' || x'0A' || '");
        while (*p) p++;
     } 
     cert++;
   }
   *p++ = '\'';
   *p++ = '\0';
   return (fcert);
}


/* Add a request record to the db.
   Returns the reqno of the new record or zero on error. */

int db_load_req(SvrReq R)
{
   char cmd[10000];
   char *fd, *fc, *fn;
   int rc, reqno=0;
   long hash;
   
   hash = fnv_hash_32(R->request);
   fd = db_fix_date(R->request_date);
   fc = db_fix_string(R->request);
   fn = db_fix_string(R->dn);
   sprintf(cmd,
         "insert into svreq (reqid,status,request_date,request,dn,rhash) "
                " values ('%s',%d,'%s',%s,%s,%d);",
               R->reqid, R->status, 
               fd, fc, fn,hash);

   rc = SQLExecDirect(h_xxx, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_abort(cmd);
      SQLFreeStmt(h_xxx, SQL_CLOSE);
      return (0);
   }

   /* Get the request number */
   strcpy(cmd,"select identity_val_local() from svreq");
   rc = SQLExecDirect(h_xxx, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_abort(cmd);
      SQLFreeStmt(h_xxx, SQL_CLOSE);
      return (0);
   }
   rc = SQLFetch(h_xxx);
   if (rc == SQL_NO_DATA_FOUND) {
      /* fprintf(stderr, "fetch eof\n"); */
      SQLFreeStmt(h_xxx, SQL_CLOSE);
      return (0);
   } else if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
      fprintf(stderr, "reqno fetch err = %d\n", rc);
      SQLFreeStmt(h_xxx, SQL_CLOSE);
      return (-1);
   }
   rc = SQLGetData(h_xxx, 1, SQL_C_LONG, &reqno, sizeof(reqno), NULL);

   SQLFreeStmt(h_xxx, SQL_CLOSE);
   return (reqno);
}


/* Add a cert record to the db.
   Returns the crtno of the new record or zero on error. */

int db_load_crt(SvrCert S)
{
   char cmd[10000];
   char *fi, *fe, *fc;
   int rc, crtno=0;
   
   fi = strdup(db_fix_date(S->issue_date));
   fe = strdup(db_fix_date(S->expire_date));
   fc = db_fix_string(S->cert);

   sprintf(cmd,
      "insert into svcrt " 
      " (reqno,status,casn,issue_date,issue_by,cert,expire_date,expno) "
                " values (%d,%d,%d,'%s','%s',%s,'%s',0);",
               S->reqno, CRT_OK, S->casn, 
               fi, S->issue_by, fc, fe);

   rc = SQLExecDirect(h_xxx, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_errprint(SQL_HANDLE_STMT, h_crt, rc);
      sql_abort(cmd);
      SQLFreeStmt(h_xxx, SQL_CLOSE);
      return (0);
   }

   /* Get the cert number */
   strcpy(cmd,"select identity_val_local() from svcrt");
   rc = SQLExecDirect(h_xxx, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_errprint(SQL_HANDLE_STMT, h_crt, rc);
      sql_abort(cmd);
      SQLFreeStmt(h_xxx, SQL_CLOSE);
      return (0);
   }
   rc = SQLFetch(h_xxx);
   if (rc == SQL_NO_DATA_FOUND) {
      /* fprintf(stderr, "fetch eof\n"); */
      SQLFreeStmt(h_xxx, SQL_CLOSE);
      return (0);
   } else if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
      fprintf(stderr, "reqno fetch err = %d\n", rc);
      SQLFreeStmt(h_xxx, SQL_CLOSE);
      return (-1);
   }
   rc = SQLGetData(h_xxx, 1, SQL_C_LONG, &crtno, sizeof(crtno), NULL);

   SQLFreeStmt(h_xxx, SQL_CLOSE);
   return (crtno);
}


/* Add a notes record to the db */

void fix_notes(char *notes)
{
  for (;*notes;notes++) if (*notes=='\'') *notes = '"';
}

int db_load_notes(int reqno, char *notes)
{
   char cmd[10000];
   char *fn;
   
   fn = db_fix_string(notes);
   fix_notes(notes);
   sprintf(cmd, "insert into svnot (reqno,notes) values (%d,%s);",
               reqno, fn); 

   rc = SQLExecDirect(h_xxx, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_errprint(SQL_HANDLE_STMT, h_not, rc);
      sql_abort(cmd);
      SQLFreeStmt(h_xxx, SQL_CLOSE);
      return (0);
   }
   SQLFreeStmt(h_xxx, SQL_CLOSE);
   return (rc==SQL_SUCCESS);
}



/* update the crt number in a request record */

int db_set_req_crtno(int reqno, int crtno)
{
   char cmd[1000];
   
   sprintf(cmd, "update svreq set (status,crtno) = (%d,%d) where reqno = %d;",
               ST_ISSUE, crtno, reqno); 

   rc = SQLExecDirect(h_xxx, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_errprint(SQL_HANDLE_STMT, h_xxx, rc);
      sql_abort(cmd);
      return (0);
   }
   SQLFreeStmt(h_xxx, SQL_CLOSE);
   return (rc==SQL_SUCCESS);
}



/* Set the status of a request record. Only some changes are allowed.
     
      REQ -> DENY
      REQ -> CANCEL
      DENY -> DELETED
      ISSUE -> RENEW
      ISSUE -> REVREQ
      REVREQ -> REVOKED
      any -> EXPIRED
   */

int db_set_req_status(int reqno, int status, char *id)
{
   char cmd[1024];
   char idsel[512];
   char pssel[512];
   int ps = ST_NONE;
   
   if (status==ST_DENY) ps = ST_REQ;
   if (status==ST_DELETED) ps = ST_DENY;
   if (status==ST_RENEW) ps = ST_ISSUE;
   if (status==ST_CANCEL) ps = ST_REQ;
   if (status==ST_REVREQ) ps = ST_ISSUE;
   if (status==ST_REVOKED) ps = ST_REVREQ;
   if (ps!=ST_NONE) sprintf(pssel,"and status=%d", ps);
   else pssel[0] = '\0';

   idsel[0] = '\0';
   if (id) sprintf(idsel," and reqid='%s'", id);

   sprintf(cmd,
    "update svreq set status=%d where reqno=%d %s %s;",
               status, reqno, pssel, idsel); 

   rc = SQLExecDirect(h_xxx, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_errprint(SQL_HANDLE_STMT, h_xxx, rc);
      sql_abort(cmd);
      return (0);
   }
   SQLFreeStmt(h_xxx, SQL_CLOSE);
   return (rc==SQL_SUCCESS);
}


/* Set the status of a cert record. Only some changes are allowed.
     
   */

int db_set_crt_status(int crtno, int status, int reqno)
{
   char cmd[1024];
   
   sprintf(cmd,
    "update svcrt set status=%d where crtno=%d and reqno=%d;",
               status, crtno, reqno); 

   rc = SQLExecDirect(h_xxx, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_errprint(SQL_HANDLE_STMT, h_xxx, rc);
      sql_abort(cmd);
      return (0);
   }
   SQLFreeStmt(h_xxx, SQL_CLOSE);
   return (rc==SQL_SUCCESS);
}



/* Set the expno of a cert record. 
     
   */

int db_set_crt_expno(int crtno, int no)
{
   char cmd[1024];
   
   sprintf(cmd,
    "update svcrt set expno=%d where crtno=%d;",
               no, crtno); 

   rc = SQLExecDirect(h_xxx, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_errprint(SQL_HANDLE_STMT, h_xxx, rc);
      sql_abort(cmd);
      return (0);
   }
   SQLFreeStmt(h_xxx, SQL_CLOSE);
   return (rc==SQL_SUCCESS);
}


/* Set the revoke date of a cert record. 
   */

int db_set_crt_revoke_date(int crtno, time_t rdate)
{
   char cmd[1024];
   char *rd;
   
   rd = db_fix_date(rdate);
   sprintf(cmd,
    "update svcrt set revoke_date='%s' where crtno=%d and status=%d;",
               rd, crtno, CRT_REVOKED); 

   rc = SQLExecDirect(h_xxx, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_errprint(SQL_HANDLE_STMT, h_xxx, rc);
      sql_abort(cmd);
      return (0);
   }
   SQLFreeStmt(h_xxx, SQL_CLOSE);
   return (rc==SQL_SUCCESS);
}


/* Change the owner of a request. 
     
   */

int db_set_req_reqid(int reqno, char *oldid, char *newid)
{
   char cmd[1024];
   
   sprintf(cmd,
    "update svreq set reqid='%s' where reqno=%d and reqid='%s';",
               newid, reqno, oldid); 

   rc = SQLExecDirect(h_xxx, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_errprint(SQL_HANDLE_STMT, h_xxx, rc);
      sql_abort(cmd);
      return (0);
   }
   SQLFreeStmt(h_xxx, SQL_CLOSE);
   return (rc==SQL_SUCCESS);
}


/* Select request record by hash value.  Tests for duplicates.

   Returns 0 if request not found.
   */


int db_check_req_hash(long hash)
{
   char cmd[1024];
   int rc;

   /* Build the select */

   sprintf(cmd, "select reqno from svreq where rhash = %d and status != %d;", 
       hash, ST_CANCEL);
   
   rc = SQLExecDirect(h_xxx, cmd, SQL_NTS);
   if (rc!=SQL_SUCCESS) {
      sql_errprint(SQL_HANDLE_STMT, h_xxx, rc);
      sql_abort(cmd);
      SQLFreeStmt(h_xxx, SQL_CLOSE);
      return (-1);
   }
   
   rc = SQLFetch(h_xxx);

   if (rc == SQL_NO_DATA_FOUND) {
      SQLFreeStmt(h_xxx, SQL_CLOSE);
      return (0);    /* Not found - good */
   } else if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
      fprintf(stderr, "fetch err = %d\n", rc);
      SQLFreeStmt(h_xxx, SQL_CLOSE);
      return (-1);
   }
   SQLFreeStmt(h_xxx, SQL_CLOSE);
   return (1);
}
