/* ldap group library */

#include <sys/types.h>


/* ---- ldap defaults: these must match the ldap schema - */

/* server hostname */
#define UWA_LDAP_HOST         "rufus02.u.washington.edu"

/* ports for plain and ssl connections */
#define UWA_LDAP_PORT          289
#define UWA_LDAP_SSL_PORT      289

/* Base of group searches */
#define UWA_LDAP_GR_BASE      "ou=groups,dc=washington,dc=edu"

/* Base of userid searches */
#define UWA_LDAP_USR_BASE     "dc=washington,dc=edu"

/* Base of courses searches */
#define UWA_LDAP_CR_BASE      "ou=courses,dc=washington,dc=edu"
/* base is actually "ou=QQQYYYY,ou=courses,..." */

/* formst to make member attribute value from userid */
#define UWA_LDAP_GR_MBRFMT   "uwnetid=%s"

/* formst to make member attribute value from eppn */
#define UWA_LDAP_GR_EPPNFMT   "eduPersonPrincipalName=%s"

/* attribute of group names */
#define UWA_LDAP_GR_NAMEATTR      "cn"

/* attribute of group members */
#define UWA_LDAP_GR_MBRATTR      "member"

/* attribute of group members who are groups */
#define UWA_LDAP_GR_GMBRATTR     "memberGroup"

/* attribute of group owners */
#define UWA_LDAP_GR_ONRATTR      "owner"

/* formst to make course attribute value from sln */
#define UWA_LDAP_CR_MBRFMT   "uwnetid=%s"

/* attribute of sln names */
#define UWA_LDAP_CR_NAMEATTR      "sln"

/* attribute of course student members */
#define UWA_LDAP_CR_STDATTR      "student"

/* attribute of course instructors members */
#define UWA_LDAP_CR_INSATTR      "instructor"

/* attribute of other course owners */
#define UWA_LDAP_CR_ONRATTR      "owner"

/* attribute of user type codes */
#define UWA_LDAP_TYPATTR      "eduPersonAffiliation"

/* local eppn domain */
#define UWA_EPPN_DEFAULT  "@washington.edu"



/* -------- ldap context ------------------------ */


typedef struct LdapCfg__ {
  char *host;
  int port;
  int sslport;
  
  char *certdb;    /* certdb */
  char *bindcrt;   /* cert for bind */
  char *bindkey;   /* key for cert bind */
  char *binddn;    /* principal */
  char *bindpw;    /* pw */
#ifdef ENABLE_KRB5
  char *k5prin;     /* k5 principal */
  char *k5ktab;     /* k5 keytab */
#endif
  int  gssapi;

  char *gr_base;       /* base for group lookup*/
  char *gr_mbrfmt;     /* format for group member attribute value (uwnetid) */
  char *gr_eppnfmt;    /* format for group member attribute value (eppn) */
  char *gr_nameattr;   /* attribute name for group name */
  char *gr_mbrattr;    /* attribute name for group user entry */
  char *gr_gmbrattr;   /* attribute name for group group entry */
  char *gr_onrattr;    /* attribute name for group owner entry */

  char *cr_base;      /* base for courses lookup */
  char *cr_mbrfmt;    /* format for course member attribute value (sln) */
  char *cr_nameattr;   /* attribute name for course sln */
  char *cr_stdattr;    /* student attribute */
  char *cr_insattr;    /* instructor attribute */
  char *cr_onrattr;    /* 'other' attribute */

  char *usr_base;      /* base for user lookup */
  char *usr_typattr;   /* attribute name for user type code */
  char *usr_snattr;    /* attribute name for user student number */
} LdapCfg_, *LdapCfg;


/* interfaces  */

LdapCfg new_ldap_cfg();
void *uw_auth_initialize(LdapCfg cfg);
void *uw_auth_initialize_k5(LdapCfg cfg);

int uw_auth_chk_in_group(void *ld, LdapCfg cfg, char *id, char *group);

int uw_auth_chk_in_course(void *ld, LdapCfg cfg, char *id, char *course, char *onr);

void *uw_auth_get_types(void *ld, LdapCfg cfg, char *id);

void uw_auth_free_types(void *types);

int uw_auth_chk_type(void *types, char *type);

void uw_auth_disconnect(void *ld);

