/* CA statistics 

   make plot and report 

 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "ezp.h"
#include "db.h"
#include "webtpl.h"


/* Start of the CA was Sat Mar  1 2003 */

#define START_OF_CA 1046505600
#define SPD (60*60*24)

int debug = 0;

int *cc = NULL;  /* concurrent cert counts */

/* plot parameters */

int cc_base = START_OF_CA;

V *newV(int steps)
{
   V *v = (V*) malloc(sizeof(V)*steps);
   memset (v, 0, sizeof(V)*steps);
   return (v);
}


int ncrt = 0;
int nrev = 0;

typedef struct LogRec__ {
  char *dn;
  time_t issue_date;
  time_t expire_date;
  time_t revoke_date;
} LogRec_, *LogRec;

/* By domain counter */

typedef struct DomCnt__ {
  struct DomCnt__ *next;
  char *dom;
  int cnt;
} DomCnt_, *DomCnt;

DomCnt_ domroot_ = {NULL, "", 0};
DomCnt domroot = &domroot_;

void step_dom(char *dom) {
  DomCnt D, L, N;
  int c = 1;

  for (D=domroot,L=NULL;D;L=D,D=D->next) if ((c=strcmp(D->dom,dom))>=0) break;

  if (c==0) D->cnt++;
  else {
    N = (DomCnt) malloc(sizeof(DomCnt_));
    N->dom = strdup(dom);
    N->cnt = 1;
    N->next = D;
    L->next = N;
    printf("dom add: %s\n", dom);
  }
}

char *getdom(char *addr) {
  char *d = strdup(addr);
  char *p;
  for (p=d;*p;p++);
  for (p--;(p>d)&&(*p!='.');p--);
  for (p--;(p>d)&&(*p!='.');p--);
  for (p--;(p>d)&&(*p!='.');p--);
  return ((*p=='.')?p+1:p);
}

/* scan log rec and partially fill in a cert record */

static int next_crt(FILE *f, LogRec L) {
   static char rec[256];
   int cn, id, ed, rd;
   int n;
   int nc;
   char *p;

   while (fgets(rec,256,f)) {
      n = sscanf(rec, "%x %d %d %d %n", &cn, &id, &ed, &rd, &nc);
      if (n!=4) continue;
      memset(L, '\0', sizeof(LogRec_));
      L->issue_date = id;
      if (L->issue_date<=0) continue;
      L->expire_date = ed;
      L->revoke_date = rd;
      if (p=strstr(rec+nc, "CN=")) {
         L->dn = p+3;
         if (p=strchr(L->dn,'/')) *p = '\0';
      } else L->dn = rec+nc;
      if (p=strchr(L->dn, '\n')) *p = '\0';
      lower_case(L->dn);
      return (1);
   }
   return (0);
}
       
   
/* scan all db records and make the plot */

int mk_cc_plot(int wd, int ht, char *title, char *pf) {

   LogRec S = (LogRec) malloc(sizeof(LogRec_));

   time_t now = time(NULL);
   int s;
   int ncc;
   int id, ed;
   V *cct; /* total */
   V *ccc; /* ua */
   V *cco; /* other */
   V *ccd;
   int i;

   EZP_Image ei;
   EZP_Graph g1;
   EZP_Color bgc, fgc;
   EZP_Color bcct, bccc, bcco;
   EZP_Style sct, scc, sco;

   FILE *dblog;
   char logrec[256];
   char *logname;

   /* accumulate data */

   ncc = (now-cc_base) / SPD;   /* one per day */
   if (debug) fprintf(stderr, "base = %d\n ncc = %d\n", cc_base, ncc);

   cct = newV(ncc+1);
   ccc = newV(ncc+1);
   cco = newV(ncc+1);
   ccd = newV(ncc+1);
   for (i=0;i<=ncc;i++) ccd[i].i = cc_base + (i*SPD);

   logname = (char*) malloc(strlen(DIRROOT)+32);
   sprintf(logname, "%s/stats/db.exp", DIRROOT);

   if (!(dblog=fopen(logname, "r"))) {
      perror(logname);
      exit (1);
   }
   while (next_crt(dblog, S)) {
        int ua = 0;
        char *p;
        if (p=strchr(S->dn,' ')) continue;
        if (S->issue_date<cc_base) S->issue_date = cc_base;
        id = (S->issue_date - cc_base) / SPD;
        ed = (S->expire_date - cc_base) / SPD;
        if (strstr(S->dn,".cac.")) ua = 1;
        if (strstr(S->dn,".u.")) ua = 1;
        for (i=id; i<ed && i<ncc; i++) {
           if (ua) ccc[i].i++;
           else cco[i].i++;
           cct[i].i++;
        }
        step_dom(getdom(S->dn));
        ncrt++;
        if (S->revoke_date && (S->expire_date>now)) nrev++;
   }
   fclose(dblog);

   /* Plot the results */

 if (pf) {
   ezp_verbose = debug;

   fprintf(stderr,"# certs:  %d \n", ncrt);

   ei = ezp_create_image (wd, ht);
   bgc = ezp_allocate_rgbcolor(ei, 255,245,235);  /* bg color */
   fgc = ezp_allocate_rgbcolor(ei, 0,0,0);        /* fg (text) color */
   bcct = ezp_allocate_rgbcolor(ei, 0,0,255);     /* total color */
   bccc = ezp_allocate_rgbcolor(ei, 34,139,34);   /* ua color */
   bcco = ezp_allocate_rgbcolor(ei, 255,0,0);     /* other color */

   if (title) ezp_image_title(ei, title, fgc);
   
   g1 = ezp_create_graph(ei, EZP_TYPE_TIME, EZP_TYPE_INT);

   ezp_graph_captions(g1, "Date", "# certificates", fgc);

   sct = ezp_create_style(EZP_STYLE_LINE, 3, 3, bcct);
   scc = ezp_create_style(EZP_STYLE_LINE, 3, 3, bccc);
   sco = ezp_create_style(EZP_STYLE_LINE, 3, 3, bcco);
   
   ezp_plot_xy(g1, ccd, cct, ncc, sct, "Total");
   ezp_plot_xy(g1, ccd, ccc, ncc, scc, "UA (cac | u)");
   ezp_plot_xy(g1, ccd, cco, ncc, sco, "other");

   ezp_save_image(ei,  EZP_IMAGE_PNG, pf);
 }

}

char *prog;
void usage()
{
   fprintf(stderr,"usage: %s [-d] [-p plotfile]\n", prog);
   exit(1);
}


#define GETARG if (++argv,--argc<0) usage()
#define ARG argv[0]

main(argc, argv)
int	argc;
char	*argv[];
{
   int	i, j, c;

   char *sum_file = NULL;

   char *plot_title = "UW Services CA certificates in use";
   char *plot_file = NULL;
   int plot_width = 500;
   int plot_height = 300;

/* parse arguments */

   prog = argv[0];
   while (--argc > 0) {
      argv++;
      if (argv[0][0]=='-') {
        switch (argv[0][1]) {

          case 'd':
             debug = 1;
             break;

          case 'p':
             switch (argv[0][2]) {
              case 'w':
                 GETARG;
                 plot_width = atoi(ARG);
                 if (plot_width<50) usage();
                 break;
              case 'h':
                 GETARG;
                 plot_height = atoi(ARG);
                 if (plot_height<50) usage();
                 break;
              case 'f':
                 GETARG;
                 plot_file = strdup(ARG);
                 break;
              case 't':
                 GETARG;
                 plot_title = strdup(ARG);
                 break;
              default: usage();
             }
             break;

          case 's':
             GETARG;
             sum_file = strdup(ARG);
             break;

          case '?':
             usage();
             exit(1);
          }
       }
   }

   mk_cc_plot(plot_width, plot_height, plot_title, plot_file);

   if (sum_file) {
      WebTemplate W = newWebTemplate();
      time_t now = time(NULL);
      char ibuf[12];
      int sumfd = open(sum_file,O_CREAT|O_TRUNC|O_RDWR);
      DomCnt D;
      char *stats_tpl;

      if (!sumfd) {
         perror(sum_file);
         exit (1);
      }

      stats_tpl = (char*) malloc(strlen(DIRROOT)+32);
      sprintf(stats_tpl, "%s/stats/stats.tpl", DIRROOT);
      WebTemplate_get_by_name(W, "sum", stats_tpl);
      free(stats_tpl);
      WebTemplate_assign(W, "DATE", ctime(&now));
      sprintf(ibuf, "%d", ncrt);
      WebTemplate_assign(W, "NCRT", ibuf);
      sprintf(ibuf, "%d", nrev);
      WebTemplate_assign(W, "NREV", ibuf);
      for (D=domroot;D;D=D->next) {
         if (D->cnt==0) continue;
         WebTemplate_assign(W, "DNAME", D->dom);
         sprintf(ibuf, "%d", D->cnt);
         WebTemplate_assign(W, "DCNT", ibuf);
         WebTemplate_parse_dynamic(W, "sum.dom");
      }
      WebTemplate_parse(W, "SUM", "sum");
      WebTemplate_set_output(W, sumfd);
      WebTemplate_set_noheader(W);
      WebTemplate_write(W, "SUM");
      close(sumfd);
   }
}


