/* library to test ownership of a host or domain */

#include <string.h>

#include "soapH.h"
/* #include "soapContactsService.nsmap" */
#include "ContactsSoapBinding.nsmap"

static char *ctgt = \
  "https://netman.cac.washington.edu/sslr/mod_soap/Net/Contacts/soap_dish";
/*
static char *ctgt = \
  "https://nmt-webdev.cac.washington.edu/sslr/mod_soap/Net/Contacts/soap_dish";
 */


static struct soap soap;
static int soap_initd = 0;

/* initialize */

static make_rand(char *fname)
{
  FILE *f = fopen(fname, "r+");
  int i;
  int buf[256];
  size_t n;
  time_t t = time(NULL);
  srandom((int)t);
  if (f) {
     if ((n=fread(buf, 256*4, 1, f))>0) {
        int r = random();
        for (i=0;i<256;i++) buf[i] = buf[i] ^ r++;
        fseek(f, 0, SEEK_SET);
        fwrite(buf, 256*4, 1, f);
     }
     fclose(f);
  }
}


static char *pname(char *file)
{
   char *pb = (char*) malloc(strlen(DIRROOT)+strlen(file)+32);
   sprintf(pb, "%s/%s", DIRROOT, file);
   return (pb);
}


static int netact_init()
{
  char *rand_file = pname("etc/.rand");
  char *key_file = pname("etc/certs.ck");
  char *ca_file = pname("etc/ca_cert");

  if (soap_initd) return (1);
  
  /* make_rand(rand_file); */
  soap_init(&soap);
  if (soap_ssl_client_context(&soap,
       SOAP_SSL_DEFAULT,
       /* SOAP_SSL_NO_AUTHENTICATION,  */
       key_file,
       "",
       ca_file,
       NULL,
       rand_file)) {
     soap_print_fault(&soap, stderr);
     return (0);
  }
  soap_initd = 1;
  return (1);
}
  
/* check the list of owners for the specified id */

int check_id(char *id, struct ArrayOf_USCORExsd_USCOREstring *str)
{
   int n,o;
   char **ptr;

   if (!str) return (0);

   n = str->__size;
   /* o = str->__offset; */

   ptr = str->__ptr;
   while (n>0) {
     if (!strcmp(*ptr,id)) return (1);
     ptr++;
     n--;
   }
   return (0);
}

/* get the owner list */

char **get_ids(struct ArrayOf_USCORExsd_USCOREstring *str)
{
   int n,o;
   char **ptr;
   char **ret;
   char **retp;

   if (!str) return (NULL);

   n = str->__size;
   /* o = str->__offset; */

   ptr = str->__ptr;

   ret = (char**) malloc(sizeof(char*)*(n+1));
   retp = ret;

   while (n>0) {
     if (*ptr) *retp++ = strdup(*ptr);
     ptr++;
     n--;
   }
   *retp = NULL;
   return (ret);
}

/* test if a host is controlled by the specified id */

int netact_test_host(char *host, char *id)
{
  char *tgt = ctgt;
  int r;
  struct ns1__get_USCOREuwnetids_USCOREfrom_USCOREhostnameResponse ret;

  if (!soap_initd) netact_init();

  memset(&ret, '\0', sizeof(ret));

  if (soap_call_ns1__get_USCOREuwnetids_USCOREfrom_USCOREhostname (
         &soap, tgt, "", host, &ret ))
	soap_print_fault(&soap,stderr);
  r = check_id(id, ret._get_USCOREuwnetids_USCOREfrom_USCOREhostnameReturn);
  return (r);
}


/* test if a domain is controlled by the specified id */

int netact_test_domain(char *domain, char *id)
{
  char *tgt = ctgt;
  int r;
  struct ns1__get_USCOREuwnetids_USCOREfrom_USCOREdomainResponse ret;
    
  if (!soap_initd) netact_init();

  memset(&ret, '\0', sizeof(ret));

  if (soap_call_ns1__get_USCOREuwnetids_USCOREfrom_USCOREdomain (
           &soap, tgt, "", domain, &ret )) soap_print_fault(&soap,stderr);
  r = check_id(id,
        ret._get_USCOREuwnetids_USCOREfrom_USCOREdomainReturn);

  return (r);
}


/* test if a name is controlled by the specified id */

int netact_test_name(char *name, char *id)
{
   char *edp, *dp;

   /* First test as hostname */

   if (netact_test_host(name, id)) return (1);

   /* Else try domains */

   if (!(edp=strrchr(name,'.'))) return (0);
   if (!(edp=strrchr(edp-1,'.'))) return (0);

   for (dp=strchr(name,'.');dp&&dp<edp;dp=strchr(dp+1,'.')) {
      if (netact_test_domain(dp+1, id)) return (1);
   }

   return (0);
}

/* get owners for a host */

char **netact_get_host_ids(char *host)
{
  char *tgt = ctgt;
  char **r;
  struct ns1__get_USCOREuwnetids_USCOREfrom_USCOREhostnameResponse ret;

  if (!soap_initd) netact_init();

  memset(&ret, '\0', sizeof(ret));

  if (soap_call_ns1__get_USCOREuwnetids_USCOREfrom_USCOREhostname (
         &soap, tgt, "", host, &ret ))
	soap_print_fault(&soap,stderr);
  r = get_ids(ret._get_USCOREuwnetids_USCOREfrom_USCOREhostnameReturn);
  return (r);
}


/* get owners of a domain */

char **netact_get_domain_ids(char *domain)
{
  char *tgt = ctgt;
  char **r;
  struct ns1__get_USCOREuwnetids_USCOREfrom_USCOREdomainResponse ret;
    
  if (!soap_initd) netact_init();

  memset(&ret, '\0', sizeof(ret));

  if (soap_call_ns1__get_USCOREuwnetids_USCOREfrom_USCOREdomain (
           &soap, tgt, "", domain, &ret )) soap_print_fault(&soap,stderr);
  r = get_ids( ret._get_USCOREuwnetids_USCOREfrom_USCOREdomainReturn);
  return (r);
}


