#!/bin/ksh

# script to retrieve the CRL
# arg="gen" will generate a new CRL

# CA root
ca=/www/ca
catool=$ca/lib/caclient
openssl=/usr/bin/openssl
tmppem=/tmp/crl.pem
log=$ca/log/gencrl.log
crlpem=$ca/services/certs/crl.pem
crlder=$ca/services/certs/crl.der


# get address of a domain
function ipaddr {
  ip="`/usr/bin/host $1|sed -n -e '
/has address/s/.*has address *//p'`"
  echo $ip
}

# see if i am master
db="`ipaddr db.certs.cac.washington.edu`"
myhost=`hostname`
me="`ipaddr $myhost`"


echo "" >> $log
date >> $log

# generate (if master) and retrieve a crl

cmd=getcrl
gen=$1
[[ $me = $db && $gen = "gen" ]] && {
   cmd=gencrl
   echo "Generating a CRL"
}

rm -f $tmppem
$catool -c $cmd -u fox > $tmppem 2>/dev/null
(( $? )) && {
  echo "$cmd fails:" >> $log
  cat $tmppem >> $log
  exit
}

# update all local copies

cp $tmppem $crlpem
$openssl crl -in $crlpem -inform pem -out $crlder -outform der

echo "Done" >> $log
