/* CA rest api  */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>

#include "webtpl.h"
#include "db.h"
#include "util.h"
#include "cassl.h"
#include "ca.h"

extern char *remote_user;  // cert cn
extern int errno;
char *errstr = NULL;

#define REQ(s) (req && !strcmp(req,s))
#define RREQ(s) (req && !strcmp(req,s) && (reqno>0))

#define BADDN_FILE "baddn.txt"

/* handle client disconnect */

void catch_pipe(int a)
{
   signal(SIGPIPE, catch_pipe);
}

/* from request parameters */

int reqno;
int crtno;
int status;

/* error exit */
static void set_status_and_exit(WebTemplate W, int err) {
  WebTemplate_assign_int(W, "STATUS", err);
  WebTemplate_parse(W, "PAGE", "page");
  WebTemplate_write(W, "PAGE");
  exit (0);
}

/* Report a database error */

static void show_db_error(WebTemplate W, char *op)
{
   char *e = (char*) db_last_error();
   char *m = (char*)malloc(strlen(e)+strlen(op)+36);
   sprintf(m, "Internal DB error (%s): %s\n", op, e);
   WebTemplate_assign(W, "EMSG", m);
   send_mail("fox@washington.edu", "CA DB error", m);
   set_status_and_exit(W, 500);
   free(m);
}
   

/* Check a request for valid dns names. 
   Returns NULL on valid, error message otherwise.  */

char *req_chk_dns(X509_REQ *xreq)
{
   char *err;
   char **alt;
   char *em;
   if (err=ssl_chk_dns(ssl_get_cn_req(xreq),0)) {
      em = (char*) malloc(strlen(err)+128);
      sprintf(em, "We cannot process the request.<br>%s", err);
      return (em);
   }
   for (alt=ssl_get_altnames_req(xreq);alt&&*alt;alt++) {
      if (err=ssl_chk_dns(*alt, 0)) {
         em = (char*) malloc(strlen(err)+128);
         sprintf(em, "We cannot process the request.<br>%s (subjectAltName)",
                            err);
         return (em);
      }
   }
   return (NULL);
}

/* Check a request for policy compliance.
   Returns NULL on valid, error message otherwise.  */

char *req_chk_policy(X509_REQ *xreq, char *id)
{
   char *err;
   char **alt;
   if (err=ssl_chk_req_dn(xreq, NULL)) return (err);
   if (err = ssl_chk_req_altnames(xreq, NULL)) {
      strncpy(err, "AN", 2);
      return (err);
   }
   return (NULL);
}


/* Process request */

main()
{
  WebTemplate W = newWebTemplate();
  char *userid = NULL;
  char *req;
  char *seqstr;
  int seq = 0;
  int s;
  char *srt;
  time_t now = time(NULL);
  int need_default = 1;
  int show_chks = 0;
  char *p;
  char *f1_pass = NULL;

  long rhash;
 
  signal(SIGPIPE, catch_pipe);

  WebTemplate_set_comments(W, "#", NULL);
  WebTemplate_add_header(W, "Cache-Control", "no-store, no-cache, must-revalidate");
  WebTemplate_get_by_name(W, "page", "page.tpl");

  WebTemplate_header(W);

  status = 0;

  /* Get arguments */

  WebTemplate_get_args(W);

  /* Get the cn from the authenticating cert */
  p = getenv("SSL_CLIENT_CERT");
  if (p && *p) {
       X509 *x509 = ssl_txt2crt(p);
       userid = ssl_get_cn(x509);
  }
  if (!userid) userid = getenv("REMOTE_USER");
     
  if (!userid) set_status_and_exit(W, 401);

  req = WebTemplate_get_arg(W, "req");

  reqno = 0;
  p = WebTemplate_get_arg(W, "rno");
  if (p && *p) {
     reqno = atoi(p);
     if (reqno) WebTemplate_assign(W, "REQNO", p);
     else WebTemplate_assign(W, "REQNO", "-invalid-");
  }
     
  WebTemplate_assign(W, "TITLE", "UWCA");
  WebTemplate_assign(W, "ROOT", URLROOT);
  WebTemplate_assign(W, "CA", "UW CA REST");

  WebTemplate_assign(W, "USER", userid);


  /* Process a revoke request. */

  if (RREQ("revoke")) {

     int rc;
     char *p;
     char *revtxt;
     SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));

     revtxt = WebTemplate_get_arg(W, "revtxt");

     if (revtxt && *revtxt) {
        char smsg[64];

        /* set revoke */
        if (!db_connect()) show_db_error(W, "connect");
        db_select_req(reqno, NULL, userid, ST_NONE, NULL);

        if ((db_get_req(R)<=0)) set_status_and_exit(W, 404);
        db_set_req_status(reqno, ST_REVREQ, userid);
        db_load_notes(reqno, revtxt);
        write_log(SERVICES_LOG, remote_user, "revreq", reqno, 0);

        set_status_and_exit(W, ST_REVREQ);
     }
  }


  /* Process a request for new cert */

  if (REQ("new")) {

     int rc;
     char *treq;
     char *dn;
     char *cn;
     char *not;
     char *f1;
     char *err;
     int renew_req = 0;
     SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));
     X509_REQ *xreq;
     FILE *errf = fopen(path_name("log/sub.log"), "a");
     time_t errt = time(NULL);
     SvrReq renR = (SvrReq) malloc(sizeof(SvrReq_));

     if (errf) fprintf(errf, "\n%s", ctime(&errt));
     if (errf) fflush(errf);

     if (!db_connect()) show_db_error(W, "connect");

     treq = WebTemplate_get_arg(W, "csr");
     if (!treq) treq =  "No request ";
     /* Add the 'begin-end' if not there */
     if (!strstr(treq,"CERTIFICATE")) {
        char *nr = (char*) malloc(strlen(treq)+98);
        strcat(strcat(strcpy(nr,"-----BEGIN CERTIFICATE REQUEST-----\n"),
                   treq),"\n-----END CERTIFICATE REQUEST-----\n");
        free(treq);
        treq = nr;
     }
     /* remove leading-trailing spaces */
     while (*treq==' ') treq++;
     s = strlen(treq);
     for (s=strlen(treq)-1;s>0;s--) {
        if ((treq[s]!=' ')&&(treq[s]!='\n')) break;
     }
     treq[s+1] = '\0';

     rhash = fnv_hash_32(treq);

     not = WebTemplate_get_arg(W, "nottxt");
     if (!not) not = "";

     if (errf) fprintf(errf, "hash=%d\nreq=%s\nnot=%s\n", rhash, treq, not);
     if (errf) fflush(errf);
     
     xreq = ssl_txt2req(treq);
     if (xreq) {
        if (errf) fprintf(errf, "req ok, dn=%s\n", ssl_get_dn_req(xreq));
        if (errf) fflush(errf);
     } else {
        if (errf) fprintf(errf, "req bogus\n");
        if (errf) fflush(errf);
     }

     if ((!xreq) || !(dn=ssl_get_dn_req(xreq))) {

        WebTemplate_assign(W,"EMSG",
             "We could not parse your form. Was that a certificate request?.");
        if (errf) fprintf(errf, "- parse failed\n");
        if (errf) fflush(errf);
        set_status_and_exit(W, 400);

     } else if (err=req_chk_dns(xreq)) {
        WebTemplate_assign(W,"EMSG", err);
        if (errf) fprintf(errf, "- invalid dns: %s\n",err);
        if (errf) fflush(errf);
        set_status_and_exit(W, 400);
        
     } else if ((rc=db_check_req_hash(rhash))>0) {

        char m[1024];

        sprintf(m, "We cannot process the request."
             "This request is a duplicate of request %d", rc);
        WebTemplate_assign(W,"EMSG",m);
        if (errf) fprintf(errf, "- duplicate\n");
        if (errf) fflush(errf);
        set_status_and_exit(W, 400);

     } else {

        /* OK, have new request */

        char *ckemsg = NULL;
        int okdn;

        if (!ckemsg) ckemsg = req_chk_policy(xreq, userid);
        okdn = ckemsg==NULL;

          char ssub[48];
          char *smsg;
          char mdt[4];
          char *nmsg = (char*) malloc(strlen(not)+512);
          char *nmsgp = nmsg;

          if (errf) fprintf(errf, "is OK - 2\n");
          if (errf) fflush(errf);
          *nmsg = '\0';
          strcpy(R->reqid, userid);
          R->status = ST_REQ;
          R->request_date = time(NULL);
          R->crtno = 0;
          strcpy(R->dn, ssl_get_cn_req(xreq));
          strcpy(R->request, treq);


          if (reqno = db_load_req(R)) {
     if (errf) fprintf(errf, "loaded - reqno=%d\n", reqno);
     if (errf) fflush(errf);
             WebTemplate_assign(W, "REQNO", int2dec(reqno));
             WebTemplate_assign(W, "CN", R->dn);
             WebTemplate_parse_dynamic(W, "page.reqno");
             R->reqno = reqno;
             db_update_owners(R);

             /* See if we have a lifetime */
             f1 = WebTemplate_get_arg(W, "lifetime");
             if (!f1) f1 = "";
        
             int l = atoi(f1);
             if (l>0 && l<=MAX_CERT_LIFE && l!=DEF_CERT_LIFE) {
                sprintf(nmsg, "Requested lifetime: %d month%c.", l, l>1?'s':'\0');
                db_load_notes(reqno, nmsg);
                if (errf) fprintf(errf, "- life: %d\n", l);
             }
             free(f1);
     if (errf) fprintf(errf, "- OK\n");
     if (errf) fflush(errf);

          } else {
             if (errf) fprintf(errf, "- load req failed\n");
             if (errf) fflush(errf);
             show_db_error(W, "loading req");
          }
          db_free_req();

          write_log(SERVICES_LOG, remote_user, "req", reqno, 0);

        }
     if (errf) fclose(errf);
     set_status_and_exit(W, ST_REQ);

  }


  /* Process a rewewal request.

     Update the status so the cert will be resigned. */

  if (RREQ("renew")) {

     char ssub[48];
     char *smsg;
     char nmsg[64];
     int rc;
     char *p, *p1;
     SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));
     SvrCert S = (SvrCert) malloc(sizeof(SvrCert_));

     need_default = 0;

     if (!db_connect()) show_db_error(W, "connect");
     
     db_select_req(reqno, NULL, NULL, ST_NONE, NULL);

     if ((db_get_req(R)<=0)) set_status_and_exit(W, 404);
     db_free_req();

     /* if lifetime set, save it now. */
     if (p=WebTemplate_get_arg(W, "lifetime")) {
         int l = atoi(p);
        if (l>0 && l<=MAX_CERT_LIFE) {
           sprintf(nmsg, "Requested lifetime: %d month%c.", l, l>1?'s':'\0');
           db_load_notes(reqno, nmsg);
        }
     }
          
     rc = db_set_req_status(R->reqno, ST_RENEW, userid);
     if (!rc) show_db_error(W, "set rs");
     WebTemplate_assign(W, "CN", R->dn);

#ifdef SENDING_MAIL
     /* send out a notice */
     sprintf(ssub,"renew %d from %s", reqno, remote_user); 
     smsg = (char*)malloc(256 + strlen(R->dn));
     sprintf(smsg,"cn = %s\n\n", R->dn); 
     if (strcmp(remote_user,"fox")) send_mail(GETS_MAIL, ssub, smsg);
     free(smsg);
#endif

     write_log(SERVICES_LOG, remote_user, "renew", reqno, 0);

     set_status_and_exit(W, ST_RENEW);
        
  }



  if (RREQ("get")) {

     int rc;
     char pt[64];
     char pl[64];
     SvrCert S = (SvrCert) malloc(sizeof(SvrCert_));
     SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));

     sprintf(pt, "Request %d", reqno);

     WebTemplate_assign(W, "PAGETITLE", pt);

     if (!db_connect()) show_db_error(W, "connect");
     
     db_select_req(reqno, NULL, NULL, 0, NULL);
     if ((db_get_req(R)<=0)) set_status_and_exit(W, 0);

     if (R->status==ST_REQ) set_status_and_exit(W, ST_REQ);
     if (R->status==ST_EXPIRED) set_status_and_exit(W, ST_EXPIRED);
     if (R->status==ST_RENEW) set_status_and_exit(W, ST_RENEW);
     if (R->status==ST_REVOKED || R->status==ST_REVREQ) set_status_and_exit(W, ST_REVOKED);

     db_select_crt(R->crtno, R->reqno ,'\0');
     if (db_get_crt(S)<=0) {
        set_status_and_exit(W, 500);
     }

     WebTemplate_assign(W, "PEM", S->cert);
     WebTemplate_assign(W, "CN", R->dn);
     WebTemplate_assign(W, "SDATE", date2mdy(S->issue_date));
     WebTemplate_assign(W, "EDATE", date2mdy(S->expire_date));
     WebTemplate_parse_dynamic(W, "page.pem");
     set_status_and_exit(W, ST_ISSUE);
  }
 
  if (RREQ("pkcs7")) {

     int rc;
     char pt[64];
     char pl[64];
     char *ret;

     SvrCert S = (SvrCert) malloc(sizeof(SvrCert_));
     SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));

     sprintf(pt, "Request %d", reqno);

     WebTemplate_assign(W, "PAGETITLE", pt);

     if (!db_connect()) show_db_error(W, "connect");
     
     db_select_req(reqno, NULL, NULL, 0, NULL);
     if ((db_get_req(R)<=0)) set_status_and_exit(W, 500);

     if (R->status==ST_REQ) set_status_and_exit(W, ST_REQ);
     if (R->status==ST_EXPIRED) set_status_and_exit(W, ST_EXPIRED);
     if (R->status==ST_RENEW) set_status_and_exit(W, ST_RENEW);
     if (R->status==ST_REVOKED || R->status==ST_REVREQ) set_status_and_exit(W, ST_REVOKED);

     db_select_crt(R->crtno, R->reqno ,'\0');
     if (db_get_crt(S)<=0) {
        set_status_and_exit(W, 500);
     }

     rc = ca_cmd(CA_PKCS7, remote_user, S->cert, 0, 0, &ret);

     if ((rc==0) && ret) {
       char *pk = strstr(ret,"-----BEGIN ");
       if (pk) {
          WebTemplate_assign(W, "PKCS7", pk);
          WebTemplate_assign(W, "CN", R->dn);
          WebTemplate_parse_dynamic(W, "page.pkcs7");
          set_status_and_exit(W, ST_ISSUE);
       }
     }
  }
 
  if (RREQ("validcn")) {
     int rc;
     char pt[64];
     char pl[64];
     SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));

     p = WebTemplate_get_arg(W, "cn");
     sprintf(pt, "CN %s", p);
     WebTemplate_assign(W, "PAGETITLE", pt);

     if (!db_connect()) show_db_error(W, "connect");
     
     db_select_req(0, p, NULL, RST_ALIVE, NULL);
     if ((db_get_req(R)<=0)) set_status_and_exit(W, 0);
     if (R->status==ST_REQ) set_status_and_exit(W, ST_REQ);
     if (R->status==ST_EXPIRED) set_status_and_exit(W, ST_EXPIRED);
     if (R->status==ST_RENEW) set_status_and_exit(W, ST_RENEW);
     if (R->status==ST_REVOKED || R->status==ST_REVREQ) set_status_and_exit(W, ST_REVOKED);
     set_status_and_exit(W, ST_ISSUE);
  }
   
  // dont actually get here.
  WebTemplate_parse(W, "PAGE", "page");
  WebTemplate_write(W, "PAGE");

}

