/* CA utility provider: management pages */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>

#include "webtpl.h"
#include "db.h"
#include "util.h"
#include "cassl.h"
#include "ca.h"

char **netact_get_host_ids(char *name);
char **netact_get_domain_ids(char *name);

#define SELECT "selected"
#define SECURE_NAME "secure.cgi"   /* Name that needs securid */
static is_secure()
{
  char *n = getenv("SCRIPT_NAME");
  if (!n) return (0);
  if (!(n=strrchr(n,'/'))) return (0);
  if (strcmp(n+1,SECURE_NAME)) return (0);
  return (1);
}

extern char *remote_user;
extern int errno;

char *errstr = NULL;

/* xsfd check and exit */

static void chk_xsec(WebTemplate W) {
   if (xsec_chk_valid_form(W)) return;
   WebTemplate_write(W, "PAGE");
   exit(1);
}


/* Check a request for policy compliance.
   Returns NULL on valid, error message otherwise.  */

char *req_chk_policy(X509_REQ *xreq, char *id)
{
   char *err;
   char **alt;
   if (err=ssl_chk_req_dn(xreq, id)) return (err);
   if (err=ssl_chk_req_altnames(xreq, id)) {
      strncpy(err, "AN", 2); 
      return (err);
   }
   return (NULL);
}

/* Report a database error */

static void show_db_error(WebTemplate W, char *op)
{
   char *e = (char*) db_last_error();
   char *m = (char*)malloc(strlen(e)+strlen(op)+36);
   sprintf(m, "Internal DB error (%s): %s\n", op, e);
   WebTemplate_assign(W, "EMSG", m);
   send_mail("fox@washington.edu", "CA DB error", m);
   free(m);
}


/* fill out the requests lines of the display */

void show_req(WebTemplate W, int reqno, char *dn, char *id, int status, char *srt)
{
   SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));
   SvrCert S = (SvrCert) malloc(sizeof(SvrCert_));
   SvrNotes N = (SvrNotes) malloc(sizeof(SvrNotes_));
   int nshow = 0;

   if (!db_connect()) show_db_error(W, "connect");
    
   if (!db_select_req(reqno, dn, id, status, srt)) {
      WebTemplate_assign(W, "EMSG", "SQL error");
      return;
   }

   while (db_get_req(R)>0) {

      if (R->status == ST_DELETED) continue;
      if (R->status == ST_CANCEL) continue;
      if (R->status == ST_EXPIRED) continue;

      // db_update_owners(R);

      WebTemplate_assign(W, "REQNO", int2dec(R->reqno));
      /* WebTemplate_assign(W, "DN", R->dn); */
      WebTemplate_assign(W, "CN", R->dn);
      WebTemplate_assign(W, "RDATE", date2mdy(R->request_date));
      WebTemplate_assign(W, "USER", R->reqid);
      WebTemplate_assign(W, "STATUS", req_status_text[R->status]);
      WebTemplate_assign(W, "SDATE", "&nbsp;");
      WebTemplate_assign(W, "EDATE", "&nbsp;");
        
      if (R->crtno) {       
         db_select_crt(R->crtno, 0, 0);
         if (db_get_crt(S)) {
            WebTemplate_assign(W, "SDATE", date2mdy(S->issue_date));
            WebTemplate_assign(W, "EDATE", date2mdy(S->expire_date));
         }
         db_free_crt();
      }

#ifdef SHOWNOTES
      db_select_notes(R->reqno);
      while (db_get_notes(N)) {
          WebTemplate_assign(W, "NOTE", N->notes);
          WebTemplate_parse_dynamic(W, "reqlist.req.notes");
      }
      db_free_notes();
#endif

      if (R->status==ST_ISSUE) WebTemplate_assign(W, "REQCS", "reqok");
      else if (R->status==ST_REQ || R->status==ST_RENEW) WebTemplate_assign(W, "REQCS", "reqip");
      else WebTemplate_assign(W, "REQCS", "reqno");

      WebTemplate_parse_dynamic(W, "reqlist.req");
      nshow++;
   }
   db_free_req();
   
   if (nshow) WebTemplate_parse_dynamic(W, "top.clicks");
   else WebTemplate_parse_dynamic(W, "reqlist.noreq");

}

void show_req_detail(WebTemplate W, int reqno, int show_chks)
{
   SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));
   SvrCert S = (SvrCert) malloc(sizeof(SvrCert_));
   SvrNotes N = (SvrNotes) malloc(sizeof(SvrNotes_));
   char *p, *dn, *cn;
   X509_REQ *xreq;
   char *ckemsg = NULL;
   int okdn = 0;
   char **alt;
   int life = 0;
   CertLife cl;
   
   if (!db_connect()) show_db_error(W, "connect");
    
   db_select_req(reqno, NULL, NULL, ST_NONE, NULL);

   while (db_get_req(R)>0) {

      xreq = ssl_txt2req(R->request);

      db_update_owners(R);
      ssl_is_windows_req(xreq);

      if (show_chks) {
         WebTemplate_assign(W, "CKID", R->reqid);
         WebTemplate_assign(W, "CKCN", ssl_get_cn_req(xreq));
         WebTemplate_parse_dynamic(W, "reqfull.dnnotice");
      }

      dn = ssl_get_dn_req(xreq);
      if (*dn=='/') dn++;
      if (p=strchr(dn,'/')) *p++ = '\0';
      WebTemplate_assign(W, "DN1", dn);
      while (p && *p) {
         if (dn=strchr(p,'/')) *dn++ = '\0';
         WebTemplate_assign(W, "DNX", p);
         WebTemplate_parse_dynamic(W, "reqfull.dn");
         p = dn;
      }
      for (alt=ssl_get_altnames_req(xreq);alt&&*alt;alt++) {
         WebTemplate_assign(W, "ALTNAME", *alt);
         WebTemplate_parse_dynamic(W, "reqfull.altname");
      }
      WebTemplate_assign(W, "DREQNO", int2dec(R->reqno));
      WebTemplate_assign(W, "DRDATE", date2mdy(R->request_date));
      WebTemplate_assign(W, "DUSER", R->reqid);
      WebTemplate_assign(W, "DSTATUS", req_status_text[R->status]);
        
      db_select_crt(0, R->reqno, 0);
      while (db_get_crt(S)) {
         if (S->status==CRT_EXPIRED) continue;
         WebTemplate_assign(W, "CASN", int2dec(S->casn));
         WebTemplate_assign(W, "CASNX", int2hex(S->casn));
         WebTemplate_assign(W, "SDATE", date2mdy(S->issue_date));
         WebTemplate_assign(W, "SIGNER", S->issue_by);
         WebTemplate_assign(W, "EDATE", date2mdy(S->expire_date));
         WebTemplate_parse_dynamic(W, "reqfull.cert");

      }
      db_free_crt();

      db_select_notes(R->reqno);
      while (db_get_notes(N)) {
         int n, rn;
         WebTemplate_assign(W, "DNOTE", fix_string_for_html(N->notes));
         WebTemplate_parse_dynamic(W, "reqfull.notes");
         n = sscanf(N->notes, "Requested lifetime: %d month", &rn);
         if (n==1 && rn>0) {
            life = rn;
         }
      }
      db_free_notes();
      if (life==0) life = DEF_CERT_LIFE;

      /* Check for policy compliance */
      ckemsg = req_chk_policy(xreq, R->reqid);
      okdn = ckemsg==NULL;
      if (!okdn) {
         if (ckemsg) WebTemplate_assign(W, "ERRMSG", ckemsg);
         WebTemplate_assign(W,"CN",ssl_get_cn_req(xreq));
         WebTemplate_parse_dynamic(W, "reqfull.baddn");
      }

      if (R->status == ST_REQ || R->status == ST_RENEW) {
         if (!okdn) WebTemplate_assign(W, "SS", SECURE_NAME);

     /* add days selector */
     for (cl=cert_lifetimes; cl->mon; cl++) {
        WebTemplate_assign_int(W, "LMON", cl->mon);
        WebTemplate_assign(W, "LTXT", cl->txt);
        WebTemplate_assign(W, "LSEL", (life==cl->mon)?SELECT:"");
        WebTemplate_parse_dynamic(W, "reqfull.sign.life");
     }

         WebTemplate_parse_dynamic(W, "reqfull.sign");
         WebTemplate_parse_dynamic(W, "reqfull.deny");
      }

      if (R->status == ST_DENY) {
         WebTemplate_parse_dynamic(W, "reqfull.delete");
      }

      if (R->status == ST_REVREQ) {
         WebTemplate_parse_dynamic(W, "reqfull.revoke");
      }


   }
   db_free_req();

   /* show other requests by this user */

/**
     WebTemplate_get_by_name(W, "reqlist", "reqlist.tpl");
     show_req(W, 0, NULL, R->reqid, ST_NONE, "2");
     WebTemplate_parse(W, "UREQ", "reqlist");
 **/

}


/* sign a request */

int sign_req(WebTemplate W, SvrReq R, int key_usage)
{
   SvrCert S = (SvrCert) malloc(sizeof(SvrCert_));

   int rc;
   char *ret = NULL;
   char *cert = NULL;
   char *dn;
   int serial;
   time_t issuedt;
   int crtno = 0;
   X509 *xcrt;
   char *p;
   int ext = 0;
   int nm = 0;
   int ndays;
   int flags = CA_FL_ALTNAMDNS;
   
   /* See if a duration was specified */

   p = WebTemplate_get_arg(W, "days");
   if (p) {
      if (!strcmp(p,"1m")) nm = 1;
      if (!strcmp(p,"3m")) nm = 3;
      if (!strcmp(p,"6m")) nm = 6;
      if (!strcmp(p,"1y")) nm = 12;
      if (!strcmp(p,"2y")) nm = 24;
      if (!strcmp(p,"3y")) nm = 36;
   }
   ndays = count_the_days(R, nm);

   if (key_usage&0xf0) ext = CA_EXT_SC;  // includes signature and encipherment usage keys
   p = WebTemplate_get_arg(W, "type");
   if (p && !strcmp(p,"code")) ext = CA_EXT_COD;  
   if (p && !strcmp(p,"sign")) ext = CA_EXT_SC;  
 
   WebTemplate_assign(W, "REQNO", int2dec(R->reqno));

   rc = ca_cmd(CA_SIGN, remote_user, R->request, ndays*100 + ext, flags, &ret);

   if ((rc==0) && ret) {
      cert = strstr(ret,"-----BEGIN CERTIFICATE-----");
      if (cert) {

         xcrt = ssl_txt2crt(ret);
         S->reqno = R->reqno;
         strcpy(S->cert, cert);
         strcpy(S->issue_by, remote_user);
         S->casn = ssl_get_sn(xcrt);
         S->issue_date = ssl_get_issue(xcrt);
         S->expire_date = ssl_get_expire(xcrt);
             
         WebTemplate_assign(W, "CN", R->dn);
         WebTemplate_assign(W, "SDATE", date2mdy(S->issue_date));
     
         crtno = db_load_crt(S);
         db_free_req();
         if (crtno) {
            char *ssub = (char*) malloc(strlen(R->dn)+48);
            rc = db_set_req_crtno(R->reqno, crtno);
            if (!rc) show_db_error(W, "upd req");
            else {
               char *umsg = (char*) malloc(strlen(U_SIGNED_MSG) + strlen(R->dn) + 64);

               /* send the user a notice */
               sprintf(ssub,"Certificate request %d (%s) signed", R->reqno, R->dn); 
               sprintf(umsg,U_SIGNED_MSG, R->dn, R->reqno, R->reqno); 
               send_owner_mail(R, ssub, umsg);
               free(umsg);

               /* send the list a notice */
               sprintf(ssub,"Req %d signed by %s", R->reqno, remote_user); 
               if (strcmp(R->reqid,"fox")) send_mail(GETS_MAIL, ssub,
                     "That's all.\n");

               write_log(SERVICES_LOG, remote_user, "sign", R->reqno, crtno);
            }
            free(ssub);

         } else errstr = ret;
      }
   } else errstr = ret;

   return (crtno);
}

/* revoke a request - revokes the certs of a request actually */

int revoke_req(WebTemplate W, SvrReq R, int flags)
{
   SvrCert S = (SvrCert) malloc(sizeof(SvrCert_));

   int rc;
   int ret;
   int serial;
   int crtno = 0;
   char *ssub = (char*) malloc(strlen(R->dn)+100);
   X509_REQ *xreq;
   char *rdn;
   time_t now = time(NULL);
   
   xreq = ssl_txt2req(R->request);
   rdn = ssl_get_dn_req(xreq);
   
   WebTemplate_assign(W, "REQNO", int2dec(R->reqno));

   db_select_crt(0, R->reqno, CRT_OK);
   while (db_get_crt(S)) {
      char *retarg = NULL;
      ret = ca_cmd(CA_REVOKE, remote_user, rdn, S->casn, flags, &retarg);
      if (retarg && !strstr(retarg,"BEGIN X509")) puts(retarg);
      if (ret==0) {
        if (db_set_crt_status(S->crtno, CRT_REVOKED, S->reqno)) {
          db_set_crt_revoke_date(S->crtno, now);
          WebTemplate_assign(W, "CASN", int2dec(S->casn));
          WebTemplate_assign(W, "CASNX", int2hex(S->casn));
          WebTemplate_parse_dynamic(W, "top.revoke");
        }
      }
   }
   db_free_crt();

   db_set_req_status(R->reqno, ST_REVOKED, NULL);

#ifdef NOTICE_OF_REVOKE
   /* send the user a notice */
   if (R->reqno) {
      char *umsg = (char*) malloc(strlen(U_REVOKE_MSG) + strlen(R->dn) + 64);
      sprintf(ssub,"Your revocation request %d (%s) completed.", R->reqno, R->dn); 
      sprintf(umsg,U_REVOKE_MSG, R->dn, R->reqno); 
      send_owner_mail(R, ssub, umsg);
      free(umsg);
   }

   /* send the list a notice */
   sprintf(ssub,"revocation %d completed by %s", R->reqno, remote_user); 
   if (strcmp(R->reqid,"fox")) send_mail(GETS_MAIL, ssub,
      "That's all.\n");
   free(ssub);
#endif

   write_log(SERVICES_LOG, remote_user, "revoke", R->reqno, 0);

   return (1);
}

#define REQ(s) (!strcmp(req,s))
#define LIST(s) (!strcmp(list,s))

main(int argc, char **argv)
{
  WebTemplate W = newWebTemplate();
  char *user;
  char *req;
  char *list;
  char *seqstr;
  int seq = 0;
  int s;
  char *p;
  char *msgstr = NULL;
  int reqno = 0;
  char *selid;
  int full_reqno = 0;
  char *srt;
  int show_chks = 0;
  char *dn;
  int flags = CA_FL_ALTNAMDNS;

  WebTemplate_set_comments(W, "#", NULL);
  WebTemplate_add_header(W, "Cache-Control",
        "no-store, no-cache, must-revalidate");
  WebTemplate_add_header(W, "Expires", "Sat, 1 Jan 2000 01:01:01 GMT");

  WebTemplate_get_args(W);
  srt = set_sort(W, "uwcasrt_a");
  WebTemplate_header(W);

  WebTemplate_get_by_name(W, "page", "page.tpl");
  WebTemplate_get_by_name(W, "top", "admtop.tpl");

  WebTemplate_assign(W, "TITLE",
       "University of Washington Certificate Authority");
  WebTemplate_assign(W, "CA", "UW Services CA");
  WebTemplate_assign(W, "PAGETITLE", "Administration");

  xsec_set_valid_form(W);

  /* process args */

  user = remote_user;
  req = WebTemplate_get_arg(W, "req");
  list = WebTemplate_get_arg(W, "list");

  p = WebTemplate_get_arg(W, "rno");
  if (p) reqno = atoi(p);
  else reqno = 0;
  if (reqno) WebTemplate_assign(W, "REQNO", p);

  selid = WebTemplate_get_arg(W, "selid");
  if (!selid) selid = "";
  while (*selid==' ') selid++;  /* ignore leading blanks */

  dn = lower_case(WebTemplate_get_arg(W, "dn"));
  if (!dn) dn = "";
  while (*dn==' ') dn++;   /* remove spaces */
  if (p=strchr(dn,' ')) *p = '\0';

  /* fixup the req string */

  if (!list) list = "active";
  if (!req) req = "-void-";
  if (REQ("Sign this request")) req = "sign";
  if (REQ("Deny this request")) req = "deny";
  if (REQ("Delete this request")) req = "delete";

  /* propagate the list mode */
  WebTemplate_assign(W, "LIST", list);
  WebTemplate_assign(W, "SELID", selid);
  WebTemplate_assign(W, "DN", dn);
  WebTemplate_assign(W, "ROOT", URLROOT);

  WebTemplate_parse_dynamic(W, "page.adminjs");
  if (strcmp(list,"active")) WebTemplate_parse_dynamic(W, "page.admin.active");
  else WebTemplate_parse_dynamic(W, "page.admin.all");
  WebTemplate_parse_dynamic(W, "page.admin");

  /* Rest of page depends on request. */

  crumb_init(W);

  /* send the log */

  if (REQ("log")) {

     WebTemplate_assign(W, "CURPAGE", "CA log");
     WebTemplate_get_by_name(W, "log", "log.tpl");
     WebTemplate_parse(W, "BODY", "log");

  }

  /* Sign a request.  

     If the request is agrees with our policy, sign now.
     else the signer must have checked the warning boxes
          and come in via the securid route. 
          If not then show the full form.
     */


  if (REQ("sign")) {

     SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));
     X509_REQ *xreq;
     int rc;
     char smsg[24];
     int okdn;

     chk_xsec(W);

     if (!db_connect()) show_db_error(W, "connect");

     db_select_req(reqno, NULL, NULL, RST_CANBLE, NULL);

     if (db_get_req(R)>0) {
       int chks = 0;
       char *ckid, *ckcn;
       int usage = 0;
       
       xreq = ssl_txt2req(R->request);
       okdn = req_chk_policy(xreq, R->reqid)==NULL;
       usage = ssl_get_keyusage_req(xreq);

       ckid = WebTemplate_get_arg(W, "ckid");
       ckcn = WebTemplate_get_arg(W, "ckcn");
       if (ckid && (!strcmp(ckid,"on")) && 
           ckcn && (!strcmp(ckcn,"on")) &&
           is_secure()) chks = 1;

       if (okdn || chks) {    /* sign */
          rc = sign_req(W, R, usage);
          if (rc) sprintf(smsg,"Request %d signed.", reqno);
          else sprintf(smsg,"Sorry, CA says \"%s\"\n", errstr);
          WebTemplate_assign(W, "MSG", smsg);
       } else {               /* show warning form */
          WebTemplate_assign(W, "SS", SECURE_NAME);
          show_chks = 1;
          req = "full";
       }

     } else WebTemplate_assign(W, "MSG", "Request not found");

     db_free_req();
  }

  /* Revoke a request.  

     The revocation has been requested by the cert owner.
     This completes the process.
     */


  if (REQ("revoke")) {

     SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));
     X509_REQ *xreq;
     int rc;
     char smsg[48];
     int okdn;

     chk_xsec(W);

     if (!db_connect()) show_db_error(W, "connect");

     db_select_req(reqno, NULL, NULL, ST_REVREQ, NULL);

     if (db_get_req(R)>0) {

       xreq = ssl_txt2req(R->request);
       okdn = req_chk_policy(xreq, remote_user)==NULL;

       rc = revoke_req(W, R, flags);
       if (rc) sprintf(smsg,"Certificates for request %d revoked.", reqno);
       else sprintf(smsg,"Sorry, CA says \"%s\"\n", errstr);
       WebTemplate_assign(W, "MSG", smsg);

     } else WebTemplate_assign(W, "MSG", "Request not found");

     db_free_req();
  }


  /* reject a request */

  if (REQ("deny")) {

     int rc;
     char smsg[300];
     char ssub[48];
     SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));
     char *dr;
     int is_ren = 0;

     chk_xsec(W);

     dr = WebTemplate_get_arg(W, "denytxt");
     if (!dr) dr = "";
     if (!*dr) {
        WebTemplate_assign(W, "PAGE",
            "You must give a reason to deny a request.");
        WebTemplate_write(W, "PAGE");
        exit(1);
     }

     if (!db_connect()) show_db_error(W, "connect");

     db_select_req(reqno, NULL, NULL, RST_CANBLE, NULL);

     if (db_get_req(R)>0) {
        db_free_req();
        if (R->status==ST_RENEW) is_ren = 1;

        db_set_req_status(reqno, is_ren?ST_ISSUE:ST_DENY, NULL);
        if (strlen(dr)>255) dr[254] = '\0';
        if (*dr) db_load_notes(reqno, dr);

        sprintf(smsg,"Request %d denied.", reqno);
        WebTemplate_assign(W, "MSG", smsg);

        /* send the user a notice */
        if (R->reqno) {
           char *umsg = (char*) malloc(strlen(U_DENIED_MSG) + strlen(R->dn) + 64);
           sprintf(ssub,"Your certificate %s denied", 
                          is_ren?"renewal":"request"); 
           sprintf(umsg,U_DENIED_MSG, R->dn, R->reqno, R->reqno); 
           send_owner_mail(R, ssub, umsg);
           free(umsg);
        }

        /* send the list a notice */
        sprintf(ssub,"request %d denied by %s", reqno, remote_user); 
        sprintf(smsg,"because: %s\n", dr); 
        if (strcmp(R->reqid,"fox")) send_mail(GETS_MAIL, ssub, smsg);

        write_log(SERVICES_LOG, remote_user, "deny", R->reqno, 0);
     } else WebTemplate_assign(W, "MSG", "Request not found");
  }

  /* delete a request */

  if (REQ("delete")) {

     int rc;
     char smsg[24];
     SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));
     char *dr;

     chk_xsec(W);

     if (!db_connect()) show_db_error(W, "connect");

     db_select_req(reqno, NULL, NULL, ST_DENY, NULL);
     if (db_get_req(R)) {

        rc = db_delete_req(reqno);
        if (!rc) show_db_error(W, "delete");
        else {
            sprintf(smsg,"Request %d deleted.", reqno);
            WebTemplate_assign(W, "MSG", smsg);
            write_log(SERVICES_LOG, remote_user, "delete", reqno, 0);
        }
     } else WebTemplate_assign(W, "EMSG", "Request not found");
  }


  /* show details of a particular request */

  if (REQ("full")) {

     int rc;
     SvrReq R = (SvrReq) malloc(sizeof(SvrReq_));
     SvrCert S = (SvrCert) malloc(sizeof(SvrCert_));
     SvrNotes N = (SvrNotes) malloc(sizeof(SvrNotes_));

     WebTemplate_assign(W, "CURPAGE", "Request detail");
     WebTemplate_get_by_name(W, "reqfull", "reqfull.tpl");

     if (reqno) show_req_detail(W, reqno, show_chks);

     WebTemplate_parse(W, "DETAIL", "reqfull");

     list = ""; /* no default list */    

  }

  /* show ownership of a domain */

  if (REQ("test")) {
     char *edp, *dp;
     int nsd;
     int nd = 0;
     char **olist;

     if (dn && *dn) {

      if (!db_connect()) show_db_error(W, "connect");

      add_crumb(W, "services/admin/", "Admin home");

        /* show host */
        WebTemplate_assign(W, "DOMTYP", "Host");
        WebTemplate_assign(W, "DOM", dn);
        olist = netact_get_host_ids(dn);
        if (olist && *olist) {
           while (*olist) {
              WebTemplate_assign(W, "ID", *olist);
              WebTemplate_parse_dynamic(W, "top.dtest.dom.id");
              olist++;
           }
           WebTemplate_parse_dynamic(W, "top.dtest.dom");
           nd++;
        }

        /* show subdomains */
        WebTemplate_assign(W, "DOMTYP", "Domain");
        if (edp=strrchr(dn,'.')) {
           for (dp=dn-1;dp&&dp<edp;dp=strchr(dp+1,'.')) {
             nsd = 0;
             WebTemplate_assign(W, "DOM", dp+1);
             olist = netact_get_domain_ids(dp+1);
             if (olist && *olist) {
               while (*olist) {
                 WebTemplate_assign(W, "ID", *olist);
                 WebTemplate_parse_dynamic(W, "top.dtest.dom.id");
                 olist++;
                 nsd++;
               }
               WebTemplate_parse_dynamic(W, "top.dtest.dom");
               nd++;
             } 
           }
        }
        if (!nd) {
           WebTemplate_assign(W, "DOMTYP", "");
           WebTemplate_assign(W, "DOM", dn);
           WebTemplate_parse_dynamic(W, "top.dtest.nodom");
        }
        WebTemplate_parse_dynamic(W, "top.dtest");
     }

     /* display requests for this domain */

     WebTemplate_assign(W, "TBLTTL", "Certificate requests");
     WebTemplate_assign(W, "SELHOST", dn);
     WebTemplate_get_by_name(W, "reqlist", "reqlist.tpl");
     WebTemplate_parse_dynamic(W, "reqlist.forhost");
     show_req(W, 0, dn, NULL, ST_NONE, srt);
     WebTemplate_parse(W, "BODY", "reqlist");

     list = "null";

  }

  /* Users requests */

  if (REQ("promote")) {
     char *id = WebTemplate_get_arg(W, "user");
     chk_xsec(W);
     if (!db_connect()) show_db_error(W, "connect");
     if (id && *id) db_set_user_status(id, US_APPROVED);
     list = "users";
  }

  if (REQ("demote")) {
     char *id = WebTemplate_get_arg(W, "user");
     chk_xsec(W);
     if (!db_connect()) show_db_error(W, "connect");
     if (id && *id) db_set_user_status(id, US_KNOWN);
     list = "users";
  }


  /* List requests */

  if (LIST("active")) {

     WebTemplate_assign(W, "TBLTTL", "Active requests");
     WebTemplate_get_by_name(W, "reqlist", "reqlist.tpl");
     if (*selid) WebTemplate_parse_dynamic(W, "reqlist.foruser");
     show_req(W, 0, NULL, selid, RST_ACTIVE, srt);
     WebTemplate_parse(W, "BODY", "reqlist");

  }
  if (LIST("all")) {

     WebTemplate_assign(W, "TBLTTL", "All requests");
     WebTemplate_get_by_name(W, "reqlist", "reqlist.tpl");
     if (*selid) WebTemplate_parse_dynamic(W, "reqlist.foruser");
     show_req(W, 0, NULL, selid, ST_NONE, srt);
     WebTemplate_parse(W, "BODY", "reqlist");

  }
  if (LIST("users")) {

     SvrUser_ U;
     WebTemplate_assign(W, "TBLTTL", "CA users");
     WebTemplate_get_by_name(W, "users", "users.tpl");

     if (!db_connect()) show_db_error(W, "connect");
     if (db_select_user(NULL)) {
        while (db_get_user(&U)) {
            WebTemplate_assign(W, "USERID", U.id);
            if (U.status==US_APPROVED)
                   WebTemplate_parse_dynamic(W, "users.user.app");
            else WebTemplate_parse_dynamic(W, "users.user.known");
            WebTemplate_parse_dynamic(W, "users.user");
        }
        db_free_user();
     } else WebTemplate_assign(W, "EMSG", db_last_error());

     WebTemplate_parse(W, "BODY", "users");

  }

  WebTemplate_parse(W, "TOPBODY", "top");

  WebTemplate_parse(W, "PAGE", "page");
  WebTemplate_write(W, "PAGE");

}

