(*  Import the UW Services CA root certificate into the system CA store
    Make a copy of the cert store, check it, update, and replace
    
    Jim Fox
    University of Washington 
    *)

set syspath to "/System/Library/Keychains/X509Anchors"
set mypath to "~/Library/Keychains/X509.tmp"
set myfile to "X509.tmp"
set crtpath to "~/Library/Keychains/UWCA.crt"


repeat 3 times

	-- See if the cert is already installed
	
	try
		do shell script "strings " & syspath & " | grep 'UW Services CA' "
		display dialog 
			"The UW Services CA is already installed on your system. " & 
			"No further action is necessary." buttons {"OK. Forget it."} default button 1
		try
			tell application "Finder"
				move file "Install_UW_CA" in desktop to trash
			end tell
		end try
		exit repeat
	end try
	
	-- Get a local copy of the cert store
	do shell script "cp " & syspath & " " & mypath
	
	-- Store the uwca cert in a temp file 
	
	set cacrt to 
		"-----BEGIN CERTIFICATE-----
MIIEBzCCA3CgAwIBAgIBADANBgkqhkiG9w0BAQQFADCBlDELMAkGA1UEBhMCVVMx
CzAJBgNVBAgTAldBMSEwHwYDVQQKExhVbml2ZXJzaXR5IG9mIFdhc2hpbmd0b24x
FDASBgNVBAsTC1VXIFNlcnZpY2VzMRcwFQYDVQQDEw5VVyBTZXJ2aWNlcyBDQTEm
MCQGCSqGSIb3DQEJARYXaGVscEBjYWMud2FzaGluZ3Rvbi5lZHUwHhcNMDMwMjI1
MTgyNTA5WhcNMzAwOTAzMTgyNTA5WjCBlDELMAkGA1UEBhMCVVMxCzAJBgNVBAgT
AldBMSEwHwYDVQQKExhVbml2ZXJzaXR5IG9mIFdhc2hpbmd0b24xFDASBgNVBAsT
C1VXIFNlcnZpY2VzMRcwFQYDVQQDEw5VVyBTZXJ2aWNlcyBDQTEmMCQGCSqGSIb3
DQEJARYXaGVscEBjYWMud2FzaGluZ3Rvbi5lZHUwgZ8wDQYJKoZIhvcNAQEBBQAD
gY0AMIGJAoGBALwCo6h4T44m+7ve+BrnEqflqBISFaZTXyJTjIVQ39ZWhE0B3Laf
bbZYju0imlQLG+MEVAtNDdiYICcBcKsapr2dxOi31Nv0moCkOj7iQueMVU4E1Tgh
YIR2I8hqixFCQIP/CMtSDail/POzFzzdVxI1pv2wRc5cL6zNwV25gbn3AgMBAAGj
ggFlMIIBYTAdBgNVHQ4EFgQUVdfBM8b6k/gnPcsgS/VajliXfXQwgcEGA1UdIwSB
uTCBtoAUVdfBM8b6k/gnPcsgS/VajliXfXShgZqkgZcwgZQxCzAJBgNVBAYTAlVT
MQswCQYDVQQIEwJXQTEhMB8GA1UEChMYVW5pdmVyc2l0eSBvZiBXYXNoaW5ndG9u
MRQwEgYDVQQLEwtVVyBTZXJ2aWNlczEXMBUGA1UEAxMOVVcgU2VydmljZXMgQ0Ex
JjAkBgkqhkiG9w0BCQEWF2hlbHBAY2FjLndhc2hpbmd0b24uZWR1ggEAMAwGA1Ud
EwQFMAMBAf8wKwYDVR0RBCQwIoYgaHR0cDovL2NlcnRzLmNhYy53YXNoaW5ndG9u
LmVkdS8wQQYDVR0fBDowODA2oDSgMoYwaHR0cDovL2NlcnRzLmNhYy53YXNoaW5n
dG9uLmVkdS9VV1NlcnZpY2VzQ0EuY3JsMA0GCSqGSIb3DQEBBAUAA4GBAIn0PNmI
JjT9bM5d++BtQ5UpccUBI9XVh1sCX/NdxPDZ0pPCw7HOOwILumpulT9hGZm9Rd+W
4GnNDAMV40wes8REptvOZObBBrjaaphDe1D/MwnrQythmoNKc33bFg9RotHrIfT4
EskaIXSx0PywbyfIR1wWxMpr8gbCjAEUHNF/
-----END CERTIFICATE-----
"
	do shell script "echo  " & cacrt & " | tr '\\015' '\\012' > " & crtpath
	
	-- Add the uwca to the tmp keystore 
	
	do shell script "certtool i " & crtpath & " k=" & myfile
	
	-- Copy the edited store back to the system store
	
	display dialog "Installing the UW Services CA Certificate.  You will need your Administrator password." buttons {"Don't Install", "Install"} default button 2
	if button returned of result is equal to "Don't Install" then exit repeat
	repeat 3 times
		try
			do shell script "cp " & mypath & " " & syspath with administrator privileges
			exit repeat
		on error emsg
			if emsg contains "password" then
				display dialog "The password was incorrect or missing." buttons {"Give up", "Try again"} default button 2
				if button returned of result is equal to "Give up" then exit repeat
			else
				display dialog emsg
				exit repeat
			end if
		end try
	end repeat
	
	-- Check that it worked
	
	try
		do shell script "strings " & syspath & " | grep 'UW Services CA' "
		-- cleanup temp files
		try
			do shell script "rm -f " & mypath & " " & crtpath
			tell application "Finder"
				move file "Install_UW_CA" in desktop to trash
			end tell
		end try
		display dialog 
			"The UW Services CA is now installed on your system. " & 
			"You must Quit and restart Safari for the change to take effect." buttons {"Restart Safari", "I'll do it later"} default button 1
		if button returned of result is equal to "Restart Safari" then
			tell application "Safari"
				quit
			end tell
			-- following step avoids bad connection errors
			try
				close application "Safari"
			end try
			tell application "Safari"
				activate
			end tell
		end if
		
		exit repeat
	end try
	
	display dialog "The installation failed." buttons {"Give up", "Try again"}
	if button returned of result is equal to "Give up" then exit repeat
	
end repeat
